import yaml
import logging
import shutil
import pandas as pd
import biocypher

import ontoweaver
from . import testing_functions

def test_extract_reconciliate_write():

    dir = "tests/simplest/"

    import_file = ontoweaver.extract_reconciliate_write(
        dir+"/biocypher_config.yaml",
        dir+"/schema_config.yaml",
        {
            dir+"data.csv": dir+"mapping.yaml"
        }
    )

    assert(import_file)

    # output_dir = testing_functions.get_latest_directory("biocypher-out")
    # assert_output_path = dir + "/assert_output"
    # testing_functions.compare_csv_files(assert_output_path, output_dir)
    # shutil.rmtree(output_dir)

if __name__ == "__main__":
    test_extract_reconciliate_write()




def test_replace():
    from . import testing_functions
    import logging
    import ontoweaver

    logging.basicConfig(level=logging.DEBUG)

    directory_name = "replace"

    expected_nodes = [
        ('gene_hugo___123<<_>><<_>>123', 'gene_hugo', {}),
        ('variant___2', 'variant', {}),
        ('variant___aAB.()C0w', 'variant', {}),
    ]

    expected_edges = [
        ('', 'variant___aAB.()C0w', 'gene_hugo___123<<_>><<_>>123', 'variant_in_gene', {})
    ]


    data_mapping = {f"tests/{directory_name}/data.csv" : f"tests/{directory_name}/mapping.yaml" }

    nodes, edges = ontoweaver.extract(data_mapping, affix="prefix", type_affix_sep='___', raise_errors=False)

    fnodes, fedges = ontoweaver.fusion.reconciliate(ontoweaver.ow2bc(nodes), ontoweaver.ow2bc(edges), reconciliate_sep=",")

    logging.debug(fnodes)
    logging.debug(fedges)
    testing_functions.assert_equals(fnodes, expected_nodes)
    testing_functions.assert_equals(fedges, expected_edges)


if __name__ == "__main__":
    test_replace()



import ontoweaver

def test_serialize():
    node = ("Source:1", "Source", {"p1":"z"})

    edge = ("Link:0", "Source:1", "Target:2", "Link", {})

    class Source(ontoweaver.base.Node):
        pass
    class Target(ontoweaver.base.Node):
        pass
    class Link(ontoweaver.base.Edge):
        @staticmethod
        def source_type():
            return Source
        @staticmethod
        def target_type():
            return Target

    serializer = ontoweaver.serialize.All()

    s = Source.from_tuple(node, serializer)
    l =   Link.from_tuple(edge, serializer)

    assert(str(s) == "Source:1Source{'p1': 'z'}")
    assert(str(l) == "Source:1Target:2Link:0Link{}")


    serializer = ontoweaver.serialize.ID()

    s = Source.from_tuple(node, serializer)
    l =   Link.from_tuple(edge, serializer)

    assert(str(s) == "Source:1")
    assert(str(l) == "Link:0")


    serializer = ontoweaver.serialize.IDLabel()

    s = Source.from_tuple(node, serializer)
    l =   Link.from_tuple(edge, serializer)

    assert(str(s) == "Source:1Source")
    assert(str(l) == "Link:0Link")


if __name__ == "__main__":
    test_serialize()



import logging

import ontoweaver

def test_fuse():
    nodes = [
        ("Source:1", "Source", {"p1":"z"}),
        ("Source:1", "Source", {"p2":"y"}), # Simple duplicate.
        ("Target:2", "Target", {}),
        ("Target:2", "Target1", {"p1":"x", "p2":"y"}),
        ("Target:2", "Target2", {"p2":"z"}),
    ]

    on_ID = ontoweaver.serialize.ID()
    congregater = ontoweaver.congregate.Nodes(on_ID)
    for n in congregater(nodes):
        pass

    as_keys  = ontoweaver.merge.string.UseKey()
    as_first = ontoweaver.merge.string.UseFirst()
    in_lists = ontoweaver.merge.dictry.Append()
    fuser = ontoweaver.fuse.Members(ontoweaver.base.Node,
            merge_ID    = as_keys,
            merge_label = as_first,
            merge_prop  = in_lists,
        )

    fusioner = ontoweaver.fusion.Reduce(fuser)
    fusioned = fusioner(congregater)

    logging.debug("Fusioned items:")
    fused = []
    for f in fusioned:
        logging.debug("  "+repr(f))
        fused.append(f)

    assert(len(fused) == 2)
    for e in fused:
        assert("p1" in e.properties)
        assert("p2" in e.properties)
        assert("y" in e.properties["p2"])

    assert(len(fuser.ID_mapping) == 0) # Only self-mappings.

    as_sets = ontoweaver.merge.string.OrderedSet(".")
    in_lists2 = ontoweaver.merge.dictry.Append(";")
    fuser2 = ontoweaver.fuse.Members(ontoweaver.base.Node,
            merge_ID    = as_keys,
            merge_label = as_sets,
            merge_prop  = in_lists2,
        )

    fusioner2 = ontoweaver.fusion.Reduce(fuser2)
    fusioned2 = fusioner2(congregater)

    logging.debug("Fusioned items:")
    fused2 = []
    for f in fusioned2:
        logging.debug("  "+repr(f))
        fused2.append(f)

    assert(len(fused2) == 2)
    for e in fused2:
        assert("p1" in e.properties)
        assert("p2" in e.properties)
        assert("y" in e.properties["p2"])


if __name__ == "__main__":
    test_fuse()



""" Adapters handling OWL documents.
"""
import rdflib
import logging

from rdflib import RDF, RDFS, OWL

from collections.abc import Iterable
from typing import Optional

from . import base
from . import iterative
from . import validate
from . import transformer

logger = logging.getLogger("ontoweaver")


class OWLtools:
    def __init__(self, graph):
        self.graph = graph
        self.seen_labels = {}

    def iri(self, subj):
        if "#" in str(subj):
            obj = str(subj).split('#')[-1]
            return obj

        elif "/" in str(subj):
            obj = str(subj).split('/')[-1] # FIXME strong assumption
            return obj

        else:
            return subj


    def label_of(self, subj):
        # First, get the label, if any; else use the IRI end tag.
        triples = list(self.graph.triples((subj, RDFS.label, None)))
        # logger.debug(f"\tLabel triples: {triples}")
        if subj in self.seen_labels:
            logger.debug(f"Label of `{subj}` already seen: `{self.seen_labels[subj]}`")
            return self.seen_labels[subj]

        elif len(triples) == 0:
            obj = self.iri(subj)
            logger.warning(f"I can't find the label of the element `{subj}'. Label guessed from IRI: {obj}")
            self.seen_labels[subj] = obj

        else:
            if len(triples) > 1:
                logger.warning(f"There is {len(triples)} elements with label `{subj}`: {';'.join(triples)}, I'll take the first one: `{triples[0]}`.")
            assert(len(triples[0]) == 3)
            obj = str(triples[0][2])
            logger.debug(f"\t\tUse label: {obj}")
            self.seen_labels[subj] = obj
        # return biocypher._misc.pascalcase_to_sentencecase(str(obj))
        return obj[0].lower() + obj[1:]


    def node_class(self, subj):
        node_class = None
        for s,p,o in self.graph.triples((subj, RDF.type, None)):
            # logger.debug(f"({self.iri(s)})-[{self.iri(p)}]->({self.iri(o)})")
            # logger.debug(f"Individual type: {node_class}")

            if o == OWL.NamedIndividual:
                continue

            if node_class:
                raise RuntimeError(
                    f"Instantiating from multiple classes is not supported, "
                    f"fix individual `{subj}`, it should not instantiate both "
                    f"`{node_class}` and `{o}`")

            if rdflib.URIRef(o) != OWL.NamedIndividual:
                node_class = o

        return node_class


class OWLAutoAdapter(base.Adapter):
    def __init__(self,
                 graph: rdflib.Graph,
                 raise_errors = True,
                 **kwargs
                 ):

        super().__init__(
            raise_errors
        )
        self.graph = graph
        logger.debug(f"OWLAutoAdapter on {len(self.graph)} input RDF triples.")

        self.get = OWLtools(self.graph)

        if kwargs:
            logger.warning(f"OWLAutoAdapter does not support mappings, but you passed: {kwargs}, I'll ignore those arguments.")


    def run(self):
        # Iterate over individuals.
        for i,indi_triple in enumerate(self.graph.triples((None,RDF.type,OWL.NamedIndividual))):
            local_nodes = []
            local_edges = []

            subj = indi_triple[0]
            logger.debug(f"{i}:({self.get.iri(subj)})")
            subj_label = self.get.label_of(subj)

            node_class = self.get.node_class(subj)
            logger.debug(f"Individual class: {node_class}")
            if not node_class:
                logger.warning(f"Individual `{subj}` has no owl:Class, I'll ignore it.")
                continue

            properties = {}
            # Gather everything about the subject individual.
            for _,rel,obj in self.graph.triples((subj, None, None)):
                logger.debug(f"\t-[{self.get.iri(rel)}]->({self.get.iri(obj)})")

                if    obj == OWL.NamedIndividual \
                  or subj == OWL.NamedIndividual \
                  or subj_label == "NamedIndividual" \
                  or  rel == RDF.type \
                  or self.get.label_of(rel) == "type":
                    logger.debug("\t\t= pass")
                    # We don't need types, they will be set by BioCypher from labels.
                    pass

                elif type(obj) == rdflib.URIRef:
                    e = base.GenericEdge(None, self.get.iri(subj), self.get.iri(obj), {}, self.get.label_of(rel))
                    logger.debug(f"\t\t= {e}")
                    local_edges.append(e)

                else:
                    logger.debug(f"\t\t= prop[{self.get.iri(rel)}] = {self.get.iri(obj)}")
                    properties[str(rel)] = str(obj)

            assert(node_class)
            n = base.Node(self.get.iri(subj), properties, self.get.label_of(node_class))
            logger.debug(f"\tnode: {n}")
            local_nodes.append(n)

            self.edges_append(local_edges)
            self.nodes_append(local_nodes)

            yield local_nodes, local_edges


class OWLAdapter(iterative.IterativeAdapter):

    def __init__(self,
            graph: rdflib.Graph,
            subject_transformer: base.Transformer,
            transformers: Iterable[base.Transformer],
            metadata: Optional[dict] = None,
            validator: Optional[validate.InputValidator] = None,
            type_affix: Optional[base.TypeAffixes] = base.TypeAffixes.suffix,
            type_affix_sep: Optional[str] = ":",
            parallel_mapping: int = 0,
            raise_errors = True
        ):

        super().__init__(
           subject_transformer,
           transformers,
           metadata,
           validator,
           type_affix,
           type_affix_sep,
           parallel_mapping,
           raise_errors
        )
        self.graph = graph
        logger.debug(f"OWLAdapter on {len(self.graph)} input RDF triples.")

        self.get = OWLtools(self.graph)


    def iterate(self):
        for i,triple in enumerate(self.graph.triples((None,RDF.type,OWL.NamedIndividual))):
            subj,t,ni = triple
            logger.debug(f"({self.get.iri(subj)})-[{self.get.iri(t)}]->({self.get.iri(ni)})")
            node_class = self.get.node_class(subj)
            logger.debug(f"\tIndividual class: ({node_class})-")
            if not node_class:
                logger.warning(f"Individual `{subj}` has no owl:Class, I'll ignore it.")
                continue

            row = {}
            # Gather everything about the subject individual.
            for _,rel,obj in self.graph.triples((subj, None, None)):
                logger.debug(f"\t-[{self.get.iri(rel)}]->({self.get.iri(obj)})")

                if obj == OWL.NamedIndividual:
                    logger.debug("\t\t= pass")

                else:
                    t = self.get.label_of(rel)
                    o = self.get.label_of(obj)
                    logger.debug(f"\t\t=> {t} : {o}")
                    if t in row:
                        self.error(f"Multiple definitions for ({subj})--[{t}]->({o}), I don't know how to handle this.", section = "OWLAdapter", exception = exceptions.InputDataError)
                    else:
                        row[t] = o

            logger.debug(f"\t= {i}th individual:")
            for  k,v in row.items():
                logger.debug(f"\t\t{k}: {v}")

            yield i,row





def test_edges_between_columns():
    from . import testing_functions
    import logging
    import ontoweaver

    logging.basicConfig(level=logging.DEBUG)

    directory_name = "edges_between_columns"

    expected_nodes = [
        ('0:variant', 'variant', {}),
        ('1:variant', 'variant', {}),
        ('2:variant', 'variant', {}),
        ('3:variant', 'variant', {}),
        ('patient1:patient', 'patient', {}),
        ('patient2:patient', 'patient', {}),
        ('patient3:patient', 'patient', {}),
        ('patient4:patient', 'patient', {}),
        ('sample1:sample', 'sample', {}),
        ('sample2:sample', 'sample', {}),
        ('sample3:sample', 'sample', {}),
        ('sample4:sample', 'sample', {})
    ]

    expected_edges = [
        ('', '0:variant', 'patient1:patient', 'patient_has_variant', {}),
        ('', '0:variant', 'sample1:sample', 'variant_in_sample', {}),
        ('', '1:variant', 'patient2:patient', 'patient_has_variant', {}),
        ('', '1:variant', 'sample2:sample', 'variant_in_sample', {}),
        ('', '2:variant', 'patient3:patient', 'patient_has_variant', {}),
        ('', '2:variant', 'sample3:sample', 'variant_in_sample', {}),
        ('', '3:variant', 'patient4:patient', 'patient_has_variant', {}),
        ('', '3:variant', 'sample4:sample', 'variant_in_sample', {}),
        ('', 'sample1:sample', 'patient1:patient', 'sample_to_patient', {}),
        ('', 'sample2:sample', 'patient2:patient', 'sample_to_patient', {}),
        ('', 'sample3:sample', 'patient3:patient', 'sample_to_patient', {}),
        ('', 'sample4:sample', 'patient4:patient', 'sample_to_patient', {})
    ]


    data_mapping = {f"tests/{directory_name}/data.csv" : f"tests/{directory_name}/mapping.yaml" }

    nodes, edges = ontoweaver.extract(data_mapping, affix="suffix")

    fnodes, fedges = ontoweaver.fusion.reconciliate(ontoweaver.ow2bc(nodes), ontoweaver.ow2bc(edges), reconciliate_sep=",")

    logging.debug(fnodes)
    logging.debug(fedges)
    testing_functions.assert_equals(fnodes, expected_nodes)
    testing_functions.assert_equals(fedges, expected_edges)

if __name__ == "__main__":
    test_edges_between_columns()



import logging
import yaml
import io
import pandas as pd

import ontoweaver

def test_singular_plural():

    logging.debug("Load data...")

    # Do not add newlines or spaces here
    # or else the parsing will be wrong.
    data = """Patient,Variant,Source
P1,V1-1,S0
P1,V1-2,S1
P2,V2-1,S2
P2,V2-2,S3"""
    csv = io.StringIO(data)
    table = pd.read_csv(csv)

    logging.debug("Load mappings...")

    plural_mapping = """
    row:
        map:
            columns:
                - Variant
            to_subject: variant
    transformers:
        - map:
            columns:
                - Patient
            to_object: patient
            via_relation: patient_has_variant
        - map:
            columns:
                - Source
            to_properties:
                - source
            for_objects:
                - variant
    """

    singular_mapping = """
    row:
        map:
            column: Variant
            to_subject: variant
    transformers:
        - map:
            column: Patient
            to_object: patient
            via_relation: patient_has_variant
        - map:
            column: Source
            to_property: source
            # Attach to subject by default.
    """

    plural_map = yaml.safe_load(plural_mapping)
    singular_map = yaml.safe_load(singular_mapping)


    logging.debug("Run the plural adapter...")
    pnodes, pedges = ontoweaver.extract_table(table, plural_map, affix="none")

    logging.debug("Run the singular adapter...")
    snodes, sedges = ontoweaver.extract_table(table, singular_map, affix="none")

    assert(list(snodes) == list(pnodes))
    assert(list(sedges) == list(pedges))

if __name__ == "__main__":
    test_singular_plural()




import logging
import yaml
import pandas as pd

import ontoweaver

def test_translate():

    directory_name = "simplest"

    logging.debug("Load mapping...")
    yaml_mapping = """
    row:
        rowIndex:
            to_subject: variant
    transformers:
        - translate:
            columns:
                - patient
            to_object: patient
            via_relation: patient_has_variant
            translations:
                A: a
                B: b
                C: c
    """

    mapping = yaml.safe_load(yaml_mapping)

    logging.debug("Load data...")
    csv_file = "tests/" + directory_name + "/data.csv"
    table = pd.read_csv(csv_file)

    logging.debug("Run the adapter...")
    nodes, edges = ontoweaver.extract_table(table, mapping, affix="none")

    assert(nodes)
    assert(edges)

    for n in nodes:
        logging.info(n)


if __name__ == "__main__":
    test_translate()



""" Functions at the root of the ``ontoweaver`` module are high-level functions useful to the end user.

    .. note::
        If you are looking to write your own code calling OntoWeaver,
        those functions are probably the only thing you will need.

    If you want to understand how OntoWeaver works internally, you may start with
    the :func:`ontoweaver.weave` function, and follow the calls from there.
"""

from typing import Tuple
from pathlib import Path
from abc import ABCMeta as ABSTRACT, abstractmethod

import os
import yaml
import copy
import rdflib
import logging
import pathlib
import warnings

import biocypher

import pandas as pd
from pandera.pandas import errors

from . import base
from . import transformer
Node = base.Node
Edge = base.Edge
Transformer = base.Transformer
Adapter = base.Adapter
All = base.All

from . import base
from . import types
from . import tabular
from . import serialize
from . import congregate
from . import merge
from . import fuse
from . import fusion
from . import exceptions
from . import make_value
from . import make_labels
from . import loader
from . import errormanager
from . import iterative
from . import xml
from . import owl
from . import mapping

logger = logging.getLogger("ontoweaver")

__all__ = [
    'tabular',
    'types',
    'transformer',
    'serialize',
    'congregate',
    'merge',
    'fuse',
    'fusion',
    'exceptions',
    'logger',
    'loader',
    'make_value',
    'make_labels',
    'iterative',
    'xml',
    'owl',
    'mapping',
    'base',
    'autoschema',
    'weave',
    'read_table_file',
    'extract_reconciliate_write',
    'load_extract',
    'extract',
    'extract_table',
    'extract_OWL',
    'reconciliate_write',
    'reconciliate',
    'write',
    'validate_input_data',
    'validate_input_data_loaded',
    'ow2bc',
]

def autoschema(
    filename_to_mappings,
    existing_schema = {},
    extended_schema_filename = "extended_schema.yaml",
    overwrite = False,
    raise_errors = True
):
    """Extend an existing_schema with what can be deduced from the given mapping files.

       Args:
           filename_to_mappings: a dictionary of the form {"filename": "mappingfile"}
           existing_schema: a filename the schema file to extend
           extended_schema_filename: the filename to which to write the extended schema
           overwrite: if True, will raise an error if extended_schema_filename already exists
           raise_errors: if True, will stop on any error, if False, will try to proceed anyway

       Returns:
           The extended_schema_filename
    """

    logger.info("Automatically generating a BioCypher schema based on the mappings...")
    supported = []
    for f2m in filename_to_mappings:
        _,with_mapping = f2m.split(':')
        if with_mapping == "automap":
            msg = "As of now, I don't know how to handle `automap` with `autoschema`."
            logger.error(msg)
            raise exceptions.ConfigError(msg)
        else:
            supported.append(with_mapping)

    vocab = base.MappingParser
    auto_schema = copy.deepcopy(existing_schema)
    for with_mapping in supported:
        logger.debug(f"\twith user file with_mapping: `{with_mapping}`")
        if '"' in with_mapping:
            with_mapping = with_mapping.strip('"')
        with open(with_mapping, 'r') as fd:
            config = yaml.full_load(fd)
            assert config, "I must have a YAML config."

        parser = mapping.YamlParser(
            config,
            validate_output = False,  # Never validate in autoschema.
            raise_errors = raise_errors,
        )
        try:
            _ = parser()
        except Exception as err:
            logger.error(f"While parsing mapping: `{with_mapping}`.")
            raise err

        logger.debug(f"Serializing schema from mapping computed from `{with_mapping}`")
        for item in parser.declared:
            logger.debug(f"\tResolve: {item}")
            if isinstance(item, base.Transformer):
                if item.multi_type_dict:
                    # This is a type/match mapping.
                    for colval,section in item.multi_type_dict.items():
                        for pred,ptype in section.items():
                            if ptype:
                                t = ptype.__name__
                                logger.debug(f"\t\tseen type: {t}")
                                if pred in vocab.k_target \
                                         + vocab.k_subject_type \
                                         + vocab.k_final_type \
                                         + vocab.k_reverse_edge:
                                    auto_schema[t] = auto_schema.get(t, {})

                                if pred in vocab.k_properties:
                                    st = auto_schema.get(t, {})
                                    st["properties"] = st.get("properties", {})
                                    for p,v in st["properties"]:
                                        st["properties"][p] = v

                elif hasattr(item, "to_property"):
                    # This is a mono-property mapping.
                    t = item.for_object
                    logger.debug(f"\t\tproperty: {t}")
                    st = auto_schema.get(t, {})
                    st["properties"] = st.get("properties", {})
                    st["properties"][item.to_property] = "str"

                elif hasattr(item, "to_properties"):
                    # This is a multi-properties mapping.
                    # logger.debug(dir(item))
                    # logger.debug(item.to_properties)
                    # logger.debug(item.for_object)
                    t = item.for_object
                    st = auto_schema.get(t, {})
                    st["properties"] = st.get("properties", {})
                    for p in item.to_properties:
                        logger.debug(f"\t\tproperty: {p}")
                        st["properties"][p] = "str"

            elif issubclass(item, base.Node):
                t = item.__name__
                logger.debug(f"\t\tnode type: {t}")
                auto_schema[t] = auto_schema.get(t, {})
                auto_schema[t]["represented_as"] = "node"
                auto_schema[t]["label_in_input"] = t
                auto_schema[t]["properties"] = auto_schema[t].get("properties", {})
                for p in item.fields():
                    if p not in auto_schema[t]["properties"]:
                        auto_schema[t]["properties"][p] = "str"
                        logger.debug(f"\t\t\tproperty: {p}")

            elif issubclass(item, base.Edge):
                t = item.__name__
                logger.debug(f"\t\tedge type: {t}")
                auto_schema[t] = auto_schema.get(t, {})
                auto_schema[t]["represented_as"] = "edge"
                auto_schema[t]["label_in_input"] = t

                if item.source_type(): # FIXME no source for edges in extended schema.
                    auto_schema[t]["source"] = item.source_type().__name__

                auto_schema[t]["target"] = []
                for target in item.target_type():
                    auto_schema[t]["target"].append(target.__name__)

                auto_schema[t]["properties"] = auto_schema[t].get("properties", {})
                for p in item.fields():
                    if p not in auto_schema[t]["properties"]:
                        # FIXME properties does not seem to be attached to edges.
                        # if isinstance(p, base.Transformer):
                        #    p = p.properties_of[t]
                        # auto_schema[t]["properties"][p] = "str"
                        # logger.debug(f"\t\t\tproperty: {p}")
                        pass

            else:
                logger.warning(f"\t\tUnknown type `{item}`, I'll just ignore it, but you may want to double-check.")

    # def prettyprint(d, indent=2):
    #    for key, value in d.items():
    #       print('\t' * indent + str(key))
    #       if isinstance(value, dict):
    #          prettyprint(value, indent+1)
    #       else:
    #          print('\t' * (indent+1) + str(value))
    # prettyprint(auto_schema)

    # Filter out empty keys.
    logger.debug(f"Filtering out empty keys")
    sch = copy.deepcopy(auto_schema)
    for t,section in sch.items():
        if not section:
            msg = "I cannot find the information related to type `{t}`, your mapping has an error."
            logger.error(msg)
            raise exceptions.ConfigError(msg)

        for pred,val in section.items():
            if pred == "properties" and val == {}:
                del auto_schema[t][pred]
            # if not val:
            #     del auto_schema[t][pred]

    logger.debug("Collapse target multi-types into their common super-type...")
    sch = copy.deepcopy(auto_schema)
    for t,section in sch.items():
        for pred,val in section.items():
            # logger.debug(f"{pred}: {val} {type(val)}")
            if pred != "target":
                continue
            else:
                if not val:
                    logger.error(f"\tEmpty {pred}: {val}")

                elif not isinstance(val, list):
                    logger.debug(f"\tTarget already set in previous schema with value: `{val}`, skipping.")

                else:
                    assert len(val) > 0
                    # logger.debug(f"{len(val)} : {val}")
                    if len(val) == 1:
                        logger.debug(f"\tThere is only one {pred}:{val}, collapsing.")
                        auto_schema[t][pred] = val[0]
                    else:
                        msg =f"\tI don't know how to make an autoschema with" \
                             f" multiple target types like: {', '.join(val)}." \
                             " Try mapping them all to their common ancestor."
                        logger.error(msg)
                        raise exceptions.AutoSchemaError(msg)

    logger.debug("Save the extended automatic schema in YAML...")
    file_exists = os.path.isfile(extended_schema_filename)
    file_writable = os.access(extended_schema_filename, os.W_OK)

    if file_exists and not file_writable:
        msg = f"Cannot write into file `{extended_schema_filename}`."
        raise exceptions.FileAccessError(msg)

    elif (file_exists and overwrite and file_writable) or (not file_exists):
        with open(extended_schema_filename, 'w') as fd:
            fd.write(yaml.dump(auto_schema))

    elif file_exists and not overwrite:
        msg = f"You asked not to overwrite `{extended_schema_filename}`, but this file exists."
        raise exceptions.FileOverwriteError(msg)

    logger.debug("Done auto schema.")
    return extended_schema_filename


def weave(biocypher_config_path, schema_path, filename_to_mapping, parallel_mapping = 0, reconciliate_sep = "|", affix = "none", type_affix_sep = ":", validate_output = False, sort_key = None, raise_errors = True, **kwargs):
    """Calls several mappings, each on the related Pandas-readable tabular data file,
       then reconciliate duplicated nodes and edges (on nodes' IDs, merging properties in lists),
       then export everything with BioCypher.
       Returns the path to the resulting import file.

       Args:
           biocypher_config_path: the BioCypher configuration file.
           schema_path: the assembling schema file.
           filename_to_mapping: a dictionary mapping data file path to the OntoWeaver mapping yaml file to extract them.
           parallel_mapping (int): Number of workers to use in parallel mapping. Defaults to 0 for sequential processing.
           reconciliate_sep (str): The separator to use for combining values in reconciliation. Defaults to None.
           affix (str): The affix to use for type inclusion. Defaults to "none".
           type_affix_sep: The character(s) separating the label from its type affix. Defaults to ":".
           validate_output: Whether to validate the output of the transformers. Defaults to False.
           raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.
           kwargs: A dictionary of arguments to pass to pandas.read_* functions.

       Returns:
           The path to the import file.
   """
    assert sort_key == None or callable(sort_key)

    logger.info("\tExtracting data...")
    nodes, edges = extract(filename_to_mapping, parallel_mapping, affix, type_affix_sep, validate_output, raise_errors, **kwargs)

    # The fusion module is independant from OntoWeaver,
    # and thus operates on BioCypher's tuples.
    logger.debug("Convert OntoWeaver elements to BioCypher tuples.")
    bc_nodes = ow2bc(nodes)
    bc_edges = ow2bc(edges)

    logger.info("\tFusing data...")
    fnodes, fedges = reconciliate(bc_nodes, bc_edges, reconciliate_sep, raise_errors)

    if sort_key:
        logger.info(f"Sort elements on: {sort_key}.")
        snodes = sorted(fnodes, key = sort_key)
        sedges = sorted(fedges, key = sort_key)
    else:
        logger.debug("Do not sort elements.")
        snodes = fnodes
        sedges = fedges

    import_file = write(snodes, sedges, biocypher_config_path, schema_path, raise_errors)

    return import_file


def read_table_file(filename, **kwargs):
    """Read a file with Pandas, using its extension to guess its format.

    If no additional arguments are passed, it will call the
    Pandas `read_*` function with `filter_na = False`, which makes empty cell
    values to be loaded as empty strings instead of NaN values.

    Args:
        filename: The name of the data file the user wants to map.
        kwargs: A dictionary of arguments to pass to pandas.read_* functions.

    Raises:
        exceptions.FeatureError: if the extension is unknown.

    Returns:
        A Pandas DataFrame.
    """

    lpf = loader.LoadPandasFile()
    data = lpf.load([filename], **kwargs)
    return data


def extract_reconciliate_write(biocypher_config_path, schema_path, data_to_mapping, parallel_mapping = 0, reconciliate_sep = "|", affix = "none", type_affix_sep = ":", validate_output = False, sort_key = None, raise_errors = True, **kwargs):
    """Deprecated alias for ``weave`` (since version 1.2).
    """
    logger.warning("The `extract_reconciliate_write` function is deprecated and will be removed in the next version, use `weave` instead.")
    return weave(biocypher_config_path, schema_path, data_to_mapping, parallel_mapping, reconciliate_sep, affix, type_affix_sep, validate_output, sort_key, raise_errors, **kwargs)


def load_extract(data, with_mapping, with_loader, parallel_mapping = 0, affix="none", type_affix_sep=":", validate_output = False, raise_errors = True, **kwargs) -> Tuple[list[Tuple], list[Tuple]]:
    """ Load the given data with the given loader, and apply the given mapping on it.

        Args:
            data: whatever data item that the with_loader is able to load.
            with_mapping: the mapping dictionary
            with_loader: the loader with which to load data
            parallel_mapping: if > 0, will run in parallel mode
            affix: whether to add a type affix
            type_affix_sep: a string for separating the ID from the type affix
            validate_output: if True, calls the validate section of the mapping, if any
            raise_errors: if True, stop at the first error, if False, try to proceed anyway
            kwargs: arguments passed to the loader function

        Returns:
            a pair of lists, the first one are nodes, the second one are egdes
    """
    logger.info(f"Use with_loader `{with_loader.__class__.__name__}` to load `{data}`")

    assert with_loader.allows([data]), "This loader cannot handle this data"
    nodes = []
    edges = []

    data = with_loader([data], **kwargs)
    mapping_options = {}

    if with_mapping == "automap":
        logger.debug("\twith auto mapping")
        mapper = {}
        mapping_options = {"automap": True}
    else:
        if isinstance(with_mapping, dict):
            logger.debug(f"\twith explicit user mapping: `{with_mapping}`")
            config = with_mapping
        else:
            assert isinstance(with_mapping, str), "I was expecting a file name as value for the data in the data_to_mapping dictionary"
            logger.debug(f"\twith user file mapping: `{with_mapping}`")
            with open(with_mapping, 'r') as fd:
                config = yaml.full_load(fd)

        parser = mapping.YamlParser(
            config,
            validate_output=validate_output,
            raise_errors = raise_errors,
        )
        mapper = parser()

    logger.debug("Instantiate the adapter...")
    adapter = with_loader.adapter(**mapping_options)(
        data,
        *mapper,
        type_affix=affix,
        type_affix_sep=type_affix_sep,
        parallel_mapping=parallel_mapping,
        raise_errors = raise_errors,
    )
    logger.info(f"Run {type(adapter).__name__}...")
    if parallel_mapping > 0:
        logger.debug(f"\tin parallel over {parallel_mapping} cores")
        adapter.__call__()
        nodes += list(adapter.nodes)
        edges += list(adapter.edges)
    else:
        logger.debug("\tsequentially")
        for ln,le in adapter():
            nodes += ln
            edges += le
    logger.debug("OK — adapter ran.")

    return nodes, edges


def extract(data_to_mapping, parallel_mapping = 0, affix="none", type_affix_sep=":", validate_output = False, raise_errors = True, **kwargs) -> Tuple[list[Tuple], list[Tuple]]:
    """
    Extracts nodes and edges from tabular data files based on provided mappings.

    Args:
        filename_to_mapping (dict): A dictionary mapping data file path to the OntoWeaver mapping yaml file to extract them.
        data_to_mapping (tuple): Tuple containing pairs of loaded Pandas DataFrames and their corresponding loaded YAML mappings.
        parallel_mapping (int): Number of workers to use in parallel mapping. Defaults to 0 for sequential processing.
        affix (str): The affix to use for type inclusion. Defaults to "none".
        type_affix_sep: The character(s) separating the label from its type affix. Defaults to ":".
        validate_output: Whether to validate the output of the transformers. Defaults to False.
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.
        kwargs: A dictionary of arguments to pass to pandas.read_* functions.

    Returns:
        tuple: Two lists of tuples containing nodes and edges.
    """

    nodes = []
    edges = []

    lpf = loader.LoadPandasFile()
    lpd = loader.LoadPandasDataframe()
    lrf = loader.LoadOWLFile()
    lrg = loader.LoadOWLGraph()

    def pairs(iterable):
        if isinstance(iterable, dict):
            return iterable.items()
        else:
            return iterable

    for d2m in pairs(data_to_mapping):
        data, mapping = d2m
        found_loader = False
        for with_loader in [lpf, lpd, lrf, lrg]:
            logger.debug(f"Trying loader: {type(with_loader).__name__}")
            if with_loader.allows([data]):
                logger.debug("  Loader allows this data type")
                found_loader = True
                try:
                    ln,le = load_extract(data, mapping, with_loader, parallel_mapping, affix, type_affix_sep, validate_output, raise_errors, **kwargs)
                except Exception as e:
                    logger.error(f"While loading `{data}` and mapping with `{mapping}`.")
                    raise e
                nodes += ln
                edges += le
                break
            else:
                logger.debug("  Loader does not allow this data type")

        if not found_loader:
            msg = f"I found no loader able to load `{data}`"
            logger.error(msg)
            raise exceptions.FeatureError(msg)

        logger.debug(f"Currently {len(nodes)} nodes and {len(edges)} edges")

    return nodes, edges


def extract_table(df: pd.DataFrame, config: dict, parallel_mapping = 0, affix = "suffix", type_affix_sep = ":", validate_output = False, raise_errors = True):
    """
    Proxy function for extracting from a table all nodes, edges and properties
    that are defined in a PandasAdapter configuration.

    Args:
        df (pd.DataFrame): The DataFrame containing the input data.
        config (dict): The configuration dictionary.
        parallel_mapping (int): Number of workers to use in parallel mapping. Defaults to 0 for sequential processing.
        module: The module in which to insert the types declared by the configuration.
        affix (str): The type affix to use (default is "suffix").
        type_affix_sep (str): The type_affix_sep to use between labels and type annotations (default is ":").
        validate_output: Whether to validate the output of the transformers. Defaults to False.
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.

    Returns:
        PandasAdapter: The configured adapter.
    """
    return extract(
        [(df,config)],
        parallel_mapping,
        affix,
        type_affix_sep,
        validate_output,
        raise_errors
    )


def extract_OWL(graph: rdflib.Graph, config: dict, parallel_mapping = 0, affix = "suffix", type_affix_sep = ":", validate_output = False, raise_errors = True):
    """
    Proxy function for extracting from a table all nodes, edges and properties
    that are defined in a PandasAdapter configuration.

    Args:
        graph (rdflib.Graph): The RDF graph containing the input data.
        config (dict): The configuration dictionary.
        parallel_mapping (int): Number of workers to use in parallel mapping. Defaults to 0 for sequential processing.
        module: The module in which to insert the types declared by the configuration.
        affix (str): The type affix to use (default is "suffix").
        type_affix_sep (str): The type_affix_sep to use between labels and type annotations (default is ":").
        validate_output: Whether to validate the output of the transformers. Defaults to False.
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.

    Returns:
        OWLAdapter: The configured adapter.
    """
    return extract(
        [(graph,config)],
        parallel_mapping,
        affix,
        type_affix_sep,
        validate_output,
        raise_errors
    )


def reconciliate_write(nodes: list[Tuple], edges: list[Tuple], biocypher_config_path: str, schema_path: str, reconciliate_sep: str = None, raise_errors = True) -> str:
    """
    Reconciliates duplicated nodes and edges, then writes them using BioCypher.

    Args:
        nodes (list): A list of nodes to be reconciliated and written.
        edges (list): A list of edges to be reconciliated and written.
        biocypher_config_path (str): the BioCypher configuration file.
        schema_path (str): the assembling schema file
        reconciliate_sep (str): The separator to use for combining values in reconciliation. Defaults to None.

        FIXME: The raise_errors parameter is currently not used downstream because the fusion classes are to be refactored with error management.
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.

    Returns:
        str: The path to the import file.
    """
    fnodes, fedges = reconciliate(nodes, edges, reconciliate_sep, raise_errors)

    import_file = write(fnodes, fedges, biocypher_config_path, schema_path, raise_errors)

    return import_file


def reconciliate(nodes: list[Tuple], edges: list[Tuple], reconciliate_sep: str = "|", raise_errors = True) -> Tuple[str]:
    """
    Reconciliates duplicated nodes and edges, then writes them using BioCypher.

    Args:
        nodes (list): A list of nodes to be reconciliated and written.
        edges (list): A list of edges to be reconciliated and written.
        reconciliate_sep (str): The separator to use for combining values in reconciliation. Defaults to None.
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.

    Returns:
        str: The path to the import file.
    """
    assert all(isinstance(n, tuple) for n in nodes), "I can only reconciliate BioCypher's tuples"
    assert all(len(n) == 3 for n in nodes), "This does not seem to be BioCypher's tuples"

    assert all(isinstance(e, tuple) for e in edges), "I can only reconciliate BioCypher's tuples"
    assert all(len(e) == 5 for e in edges), "This does not seem to be BioCypher's tuples"

    logging.info("Fuse duplicated nodes and edges...")
    fnodes, fedges = fusion.reconciliate(nodes, edges, reconciliate_sep = reconciliate_sep, raise_errors = raise_errors)
    logger.debug(f"OK, {len(fnodes)} nodes and {len(fedges)} edges after fusion")

    return fnodes,fedges


def write(nodes: list[Tuple], edges: list[Tuple], biocypher_config_path: str, schema_path: str, raise_errors = True) -> str:
    """
    Reconciliates duplicated nodes and edges, then writes them using BioCypher.

    Args:
        nodes (list): A list of nodes to be reconciliated and written.
        edges (list): A list of edges to be reconciliated and written.
        biocypher_config_path (str): the BioCypher configuration file.
        schema_path (str): the assembling schema file
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.

    Returns:
        str: The path to the import file.
    """
    if not nodes and not edges:
        msg = "There is no node or edge to write."
        logger.warning(msg)
        raise RuntimeError(msg)
    else:
        logging.info("Export the graph...")
        bc = biocypher.BioCypher(
            biocypher_config_path = biocypher_config_path,
            schema_config_path = schema_path
        )

        if nodes:
            bc.write_nodes(nodes)
        if edges:
            bc.write_edges(edges)
        #bc.summary()
        import_file = bc.write_import_call()
        logging.info("OK")

        return import_file


def validate_input_data(filename_to_mapping: dict, raise_errors = True, **kwargs) -> bool:
    """
    Validates the data files based on provided rules in configuration.

    Args:
        filename_to_mapping (dict): a dictionary mapping data file path to the OntoWeaver mapping yaml file.
        raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.
        kwargs: A dictionary of arguments to pass to pandas.read_* functions.

    Returns:
        bool: True if the data is valid, False otherwise.
    """

    assert isinstance(filename_to_mapping, dict) # data_file => mapping_file

    for data_file, mapping_file in filename_to_mapping.items():
        table = read_table_file(data_file, **kwargs)

        with open(mapping_file, 'r') as fd:
            yaml_mapping = yaml.full_load(fd)

        validator = mapping.YamlParser(
            yaml_mapping, types,
            raise_errors=raise_errors
        )._get_input_validation_rules()

        return validate_input_data_loaded(table, validator)


def validate_input_data_loaded(dataframe, validator, raise_errors = True) -> bool:
    """
    Validates the data files based on provided rules in configuration.

    Args:
         dataframe: The loaded pandas DataFrame to validate.
         validator: The mapping object to use for validation.
         raise_errors: Whether to raise errors encountered during the mapping, and stop the mapping process. Defaults to True.

    Returns:
        bool: True if the data is valid, False otherwise.
    """

    # try:
    #     validator(dataframe)
    #     return True
    # except errors.SchemaErrors as exc:
    #     logger.error(f"Validation failed for {exc.failure_cases}.")
    #     return False
    # except Exception as e:
    #     logger.error(f"An unexpected error occurred: {e}")
    #     return False
    validator.make_raise_warnings()
    mgr = errormanager.ErrorManager(raise_errors)

    return validator(dataframe)


def ow2bc(ow_elements):
    """ Convert OntoWeaver objects into BioCypher tuples

        This is useful for converting whatever gets out of OntoWeaver's
        ``weave`` or ``extract*`` functions into tuples that can be
        fed to the fusion module or to BioCypher.
    """
    return [e.as_tuple() for e in ow_elements]




def test_reverse_edge():
    from . import testing_functions
    import logging
    import ontoweaver

    logger = logging.getLogger("ontoweaver")
    logger.setLevel(logging.DEBUG)

    directory_name = "reverse_edge"

    expected_nodes = [
        ('0:variant', 'variant', {'whatever': 'A0', 'database_name': 'my_database'}),
        ('A:disease', 'disease', {'whatever': 'A0', 'something': 'Whatever it is', 'database_name': 'my_database'}),
        ('1:variant', 'variant', {'whatever': 'B1', 'database_name': 'my_database'}),
        ('B:patient', 'patient', {'something': 'Whatever it is', 'database_name': 'my_database'}),
        ('2:variant', 'variant', {'whatever': 'C2', 'database_name': 'my_database'}),
        ('C:oncogenicity', 'oncogenicity', {'database_name': 'my_database'}),
    ]

    expected_edges = [
        ('', '0:variant', 'A:disease', 'variant_to_disease', {'something': 'Whatever it is', 'database_name': 'my_database'}),
        ('', '1:variant', 'B:patient', 'patient_has_variant', {'database_name': 'my_database'}),
        ('', 'B:patient', '1:variant', 'variant_of_patient', {'whatever': 'B1', 'database_name': 'my_database'}),
        ('', '2:variant', 'C:oncogenicity', 'variant_to_oncogenicity', {'whatever': 'C2', 'database_name': 'my_database'}),
        ('', 'C:oncogenicity', '2:variant', 'oncogenicity_of_variant', {'something': 'Whatever it is', 'database_name': 'my_database'}),
    ]

    data_mapping = {f"tests/{directory_name}/data.csv": f"tests/{directory_name}/mapping.yaml"}

    nodes, edges = ontoweaver.extract(data_mapping, affix="suffix")

    fnodes, fedges = ontoweaver.fusion.reconciliate(ontoweaver.ow2bc(nodes), ontoweaver.ow2bc(edges), reconciliate_sep=",")

    logging.debug(fnodes)
    logging.debug(fedges)
    testing_functions.assert_equals(fnodes, expected_nodes)
    testing_functions.assert_equals(fedges, expected_edges)


if __name__ == "__main__":
    test_reverse_edge()



def DISABLED_test_oncokb():
    from . import testing_functions
    import logging
    import ontoweaver

    logging.basicConfig(level=logging.DEBUG)

    directory_name = "oncokb"

    assert_nodes = [('0:variant', 'variant', {'timestamp': '2023-02-27 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'MET amplification results from the gain of the MET gene on chromosome 7q31 and is found in various cancers, including melanoma, lung and renal cancers (PMID: 28481359). Often, this amplification leads to overexpression of the MET protein, which has been demonstrated to contribute to tumorigenesis by increasing downstream pathway activation, proliferation, invasiveness, and angiogenesis (PMID: 16461907, 18077425, 17463250, 19117057, 22869872,17667909). MET amplification has also been shown to contribute to resistance to tyrosine kinase inhibitors, including gefitinib and erlotinib and more recently osimertinib, as shown by sustained pathway activation and proliferation in the presence of drug (PMID: 18093943, 17463250, 30073261). In vitro proliferation assays of cell lines with MET amplifications show sensitivity to the MET-specific kinase inhibitor, PHA-665752 (PMID: 17463250, 16461907).', 'variant_summary': 'MET amplification is known to be oncogenic.'}),
                    ('33:patient', 'patient', {}),
                    ('MET:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4233', 'gene_summary': 'MET, a receptor tyrosine kinase, is recurrently altered by mutation, amplification and rarely altered by gene translocation in various cancer types.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'While crizotinib is NCCN-compendium listed for the treatment of patients with high-level MET-amplified non-small cell lung cancer, its clinical utility in patients with MET-amplified high-grade serous ovarian cancer is unknown.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('30073261:publication', 'publication', {}),
                    ('16461907:publication', 'publication', {}),
                    ('19117057:publication', 'publication', {}),
                    ('17667909:publication', 'publication', {}),
                    ('22869872:publication', 'publication', {}),
                    ('18077425:publication', 'publication', {}),
                    ('18093943:publication', 'publication', {}),
                    ('28481359:publication', 'publication', {}),
                    ('17463250:publication', 'publication', {}),
                    ('Crizotinib:drug', 'drug', {}),
                    ('Telisotuzumab Vedotin:drug', 'drug', {}),
                    ('Capmatinib:drug', 'drug', {}),
                    ('Tepotinib:drug', 'drug', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('91:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('33:patient', 'patient', {}),
                    ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
                    ('LEVEL_3B:oncokb_evidence_level', 'oncokb_evidence_level', {}),
                    ('1:variant', 'variant', {'timestamp': '2023-06-22 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'BRAF amplification is a rare event in cancer and has been found to occur at very low frequency in melanoma and glioma (cBioPortal, MSKCC, April 2017). It has been demonstrated that amplification of wildtype BRAF in melanoma cells leads to overexpression of the protein and constitutive activation of MAPK pathway, demonstrated as a BRAF-dependent increase in ERK phosphorylation, as well as an increase in cell proliferation (PMID: 15467732). BRAF overexpression has been associated with increased proliferation in gliomas (PMID: 25618114). Amplification of BRAF V600E allele has been described as a mechanism of resistance to MEK/RAF inhibitors in melanoma via MAPK pathway hyperactivation (PMID: 22395615, 24055054, 21098728).', 'variant_summary': 'BRAF amplification is known to be oncogenic.'}),
                    ('67:patient', 'patient', {}),
                    ('BRAF:gene_hugo', 'gene_hugo', {'entrez_gene_id': '673', 'gene_summary': 'BRAF, an intracellular kinase, is frequently mutated in melanoma, thyroid and lung cancers among others.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with BRAF-amplified high-grade serous ovarian cancer.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('24055054:publication', 'publication', {}),
                    ('25618114:publication', 'publication', {}),
                    ('22395615:publication', 'publication', {}),
                    ('21098728:publication', 'publication', {}),
                    ('15467732:publication', 'publication', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('4:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('67:patient', 'patient', {}),
                    ('2:variant', 'variant', {'timestamp': '2020-11-16 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'EZH2 amplification results in overexpression of the protein. This alteration has been found in several tumor types and has been implicated as a marker of poor prognosis in various solid tumors (PMID: 12374981, 14500907, 21837672, 16330673, 16330673). EZH2 overexpression in natural killer and natural killer T-cell lymphoma cell lines resulted in a growth advantage compared to wildtype (PMID: 23529930).', 'variant_summary': 'EZH2 amplification is known to be oncogenic.'}),
                    ('88:patient', 'patient', {}),
                    ('EZH2:gene_hugo', 'gene_hugo', {'entrez_gene_id': '2146', 'gene_summary': 'EZH2, an epigenetic modifier, is altered by mutation and/or overexpression in solid tumors and hematologic malignancies.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with EZH2-amplified high-grade serous ovarian cancer.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('16330673:publication', 'publication', {}),
                    ('12374981:publication', 'publication', {}),
                    ('23529930:publication', 'publication', {}),
                    ('14500907:publication', 'publication', {}),
                    ('21837672:publication', 'publication', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('92:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('88:patient', 'patient', {}),
                    ('3:variant', 'variant', {'timestamp': '2023-02-15 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'Deletion of CDKN2A results in loss of the p16/INK4A and p14/ARF proteins. CDKN2A deletion has been found in head and neck squamous cell carcinoma, glioma, bladder carcinoma, leukemia and melanoma and is often associated with poorer clinical outcomes (PMID: 15495191, 26516359, 10999737). Deletion of CDKN2A in mouse models of malignant mesothelioma resulted in accelerated tumorigenesis compared to wildtype (PMID: 21526190). In addition, knockdown of CDKN2A in an HRAS G12V mouse model resulted in the induction of high-grade endometrial stromal sarcoma (PMID: 28982163).', 'variant_summary': 'CDKN2A deletion is known to be oncogenic.'}),
                    ('60:patient', 'patient', {}),
                    ('CDKN2A:gene_hugo', 'gene_hugo', {'entrez_gene_id': '1029', 'gene_summary': 'The CDKN2A gene encodes two proteins, p16INK4A and p14ARF, that regulate the cell growth and survival. CDKN2A is altered by mutation and/or deletion in a broad range of solid and hematologic cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'Laboratory data suggest that cancer cells with loss-of-function alterations of CDKN2A may be sensitive to CDK4/6 inhibitors such as palbociclib, ribociclib and abemaciclib.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('21526190:publication', 'publication', {}),
                    ('28982163:publication', 'publication', {}),
                    ('26516359:publication', 'publication', {}),
                    ('15495191:publication', 'publication', {}),
                    ('10999737:publication', 'publication', {}),
                    ('Palbociclib:drug', 'drug', {}),
                    ('Ribociclib:drug', 'drug', {}),
                    ('Abemaciclib:drug', 'drug', {}),
                    ('Loss-of-function:functional_effect', 'functional_effect', {}),
                    ('37:sample', 'sample', {}),
                    ('Deletion:alteration', 'alteration', {}),
                    ('60:patient', 'patient', {}),
                    ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
                    ('LEVEL_4:oncokb_evidence_level', 'oncokb_evidence_level', {}),
                    ('4:variant', 'variant', {'timestamp': '2019-02-06 00:00:00', 'version': 'v4.10', 'variant_summary': 'ETV6 amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('32:patient', 'patient', {}),
                    ('ETV6:gene_hugo', 'gene_hugo', {'entrez_gene_id': '2120', 'gene_summary': 'ETV6, a transcription factor, is frequently altered by chromosomal rearrangement in hematologic malignancies.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with ETV6-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('15:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('32:patient', 'patient', {}),
                    ('5:variant', 'variant', {'timestamp': '2018-08-01 00:00:00', 'version': 'v4.10', 'variant_summary': 'ETNK1 amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('40:patient', 'patient', {}),
                    ('ETNK1:gene_hugo', 'gene_hugo', {'entrez_gene_id': '55500', 'gene_summary': 'ETNK1, a kinase involved in phospholipid biosynthesis, is recurrently altered by mutation in BCR-ABL negative CML and CMML.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with ETNK1-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('43:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('40:patient', 'patient', {}),
                    ('6:variant', 'variant', {'timestamp': '2021-11-18 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'KRAS amplification results in overexpression of the protein. This mutation has been found in germ cell tumors and esophagogastric and ovarian cancer (cBioPortal, Zehir, A et al. Nature Medicine, 2017). In vitro studies have demonstrated that this mutation confers resistance to the tyrosine kinase inhibitor crizotinib, EGFR inhibitors cetuximab or panitumumab, MET inhibitors PHA-665752 or JNJ38877605 and MEK inhibitors AZD6244 or GSK1120212 in patient-derived lung cancer cell line and xenograft models, colorectal cancer cell line models, MET-amplified cell line models and gastric cancer cell line models (PMID: 29808010, 30072474, 23404247, 20841479). Analyses of 475 patients with endometrial cancer demonstrated that KRAS amplification is associated with metastasis (PMID: 23099803).', 'variant_summary': 'KRAS amplification is likely oncogenic.'}),
                    ('68:patient', 'patient', {}),
                    ('KRAS:gene_hugo', 'gene_hugo', {'entrez_gene_id': '3845', 'gene_summary': 'KRAS, a GTPase which functions as an upstream regulator of the MAPK pathway, is frequently mutated in various cancer types including lung, colorectal and pancreatic cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with KRAS-amplified high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('23099803:publication', 'publication', {}),
                    ('29808010:publication', 'publication', {}),
                    ('30072474:publication', 'publication', {}),
                    ('20841479:publication', 'publication', {}),
                    ('23404247:publication', 'publication', {}),
                    ('Likely Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('76:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('68:patient', 'patient', {}),
                    ('7:variant', 'variant', {'timestamp': '2021-02-17 00:00:00', 'version': 'v4.10', 'variant_summary': 'NTRK3 amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('65:patient', 'patient', {}),
                    ('NTRK3:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4916', 'gene_summary': 'NTRK3, a receptor tyrosine kinase, is altered by gene fusion in various cancer types.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with NTRK3-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('46:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('65:patient', 'patient', {}),
                    ('8:variant', 'variant', {'timestamp': '2019-02-04 00:00:00', 'version': 'v4.10', 'variant_summary': 'IDH2 amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('35:patient', 'patient', {}),
                    ('IDH2:gene_hugo', 'gene_hugo', {'entrez_gene_id': '3418', 'gene_summary': 'IDH2, a cell metabolism enzyme, is recurrently mutated in various cancer types including acute myeloid leukemia, glioblastoma, and cholangiocarcinoma.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with IDH2-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('13:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('35:patient', 'patient', {}),
                    ('9:variant', 'variant', {'timestamp': '2019-01-30 00:00:00', 'version': 'v4.10', 'variant_summary': 'MAF amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('75:patient', 'patient', {}),
                    ('MAF:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4094', 'gene_summary': 'MAF, a transcription factor, is recurrently altered by rearrangement in multiple myeloma.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with MAF-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('85:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('75:patient', 'patient', {}),
                    ('10:variant', 'variant', {'timestamp': '2023-07-05 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'The mutation effect description for truncating mutations in BRCA1 is: Truncating mutations in BRCA1 can lead to varying C-terminally truncated proteins that results in aberrant protein folding, contributing to loss of BRCA1 protein function. Human breast cancer cell lines that contain a truncating mutation in BRCA1 display elevated levels of aneuploidy and impaired DNA damage response. In addition, truncating mutations have been shown to induce aberrant protein localization, which may impact the interaction of important binding partners (PMID: 20608970). Mouse models of BRCA1 truncating mutations develop cancer, including mammary carcinomas, lymphomas and ovarian carcinomas (PMID: 11358863, 12483515, 12947386).', 'variant_summary': 'The BRCA1 Y1463* is a truncating mutation in a tumor suppressor gene, and therefore is likely oncogenic.'}),
                    ('61:patient', 'patient', {}),
                    ('BRCA1:gene_hugo', 'gene_hugo', {'entrez_gene_id': '672', 'gene_summary': 'BRCA1, a tumor suppressor involved in the DNA damage response, is mutated in various cancer types.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'The PARP inhibitors olaparib, rucaparib and niraparib are FDA-approved for the treatment of patients with ovarian, fallopian tube, and primary peritoneal cancer with deleterious germline or somatic BRCA1 or 2 mutations (olaparib and niraparib are also approved for suspected deleterious mutations in this setting).'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('11358863:publication', 'publication', {}),
                    ('20608970:publication', 'publication', {}),
                    ('12483515:publication', 'publication', {}),
                    ('12947386:publication', 'publication', {}),
                    ('Olaparib:drug', 'drug', {}),
                    ('Rucaparib:drug', 'drug', {}),
                    ('Niraparib:drug', 'drug', {}),
                    ('Talazoparib:drug', 'drug', {}),
                    ('Likely Loss-of-function:functional_effect', 'functional_effect', {}),
                    ('8:sample', 'sample', {}),
                    ('Y1463*:alteration', 'alteration', {'alteration_consequence': 'stop_gained', 'protein_start': '1463.0', 'protein_end': '1463.0'}),
                    ('61:patient', 'patient', {}),
                    ('LEVEL_Fda2:fda_evidence_level', 'fda_evidence_level', {}),
                    ('LEVEL_1:oncokb_evidence_level', 'oncokb_evidence_level', {}),
                    ('11:variant', 'variant', {'timestamp': '2023-01-24 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'The mutation effect description for truncating mutations in TP53 is: Truncating mutations of TP53 occur throughout the gene and lead to the production of several C-terminally truncated protein forms. These alterations are predicted to be inactivating and are associated with poor prognosis (PMID: 11900253, 11753428, 16007150, 21467160, 19336573). Experimental studies have revealed that truncating mutations promote cancer cell proliferation, survival and metastasis, since ectopic expression of these mutations in melanoma cells increased cell motility and tumor formation in vivo. This was due in part to aberrant localization of truncated proteins to the mitochondria, regulating genes involved in cell survival, including CypD (PMID: 27759562).', 'variant_summary': 'There is no available functional data about the TP53 X125_splice alteration (last reviewed on 01/24/2023). However, it has been identified as a statistically significant hotspot and is likely to be oncogenic.'}),
                    ('11:patient', 'patient', {}),
                    ('TP53:gene_hugo', 'gene_hugo', {'entrez_gene_id': '7157', 'gene_summary': 'TP53, a tumor suppressor in the DNA damage pathway, is the most frequently mutated gene in cancer.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with TP53 X125_splice altered high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('19336573:publication', 'publication', {}),
                    ('27759562:publication', 'publication', {}),
                    ('16007150:publication', 'publication', {}),
                    ('11753428:publication', 'publication', {}),
                    ('11900253:publication', 'publication', {}),
                    ('21467160:publication', 'publication', {}),
                    ('Likely Loss-of-function:functional_effect', 'functional_effect', {}),
                    ('1:sample', 'sample', {}),
                    ('X125_splice:alteration', 'alteration', {'alteration_consequence': 'splice_region_variant', 'protein_start': '125.0', 'protein_end': '125.0'}),
                    ('11:patient', 'patient', {}),
                    ('12:variant', 'variant', {'timestamp': '2019-01-30 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'The mutation effect description for truncating mutations in BCOR is: Truncating mutations in BCOR occur as both germline (oculofaciocardiodental or OFCD syndrome) and somatic (acute myeloid leukemia (AML) myelodysplastic syndrome, retinoblastoma) mutations in human disease, leading to reduced protein expression (PMID: 22012066, 24047651, 25550361, 22237022). Experimental studies using AML patient samples suggest loss of BCOR protein enhances cell proliferation and myeloid differentiation (PMID: 26847029).', 'variant_summary': 'The BCOR S1704Kfs*16 is a truncating mutation in a tumor suppressor gene, and therefore is likely oncogenic.'}),
                    ('77:patient', 'patient', {}),
                    ('BCOR:gene_hugo', 'gene_hugo', {'entrez_gene_id': '54880', 'gene_summary': 'BCOR, a transcriptional repressor, is altered in various solid and hematologic malignancies including acute myeloid leukemia.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with BCOR S1704Kfs*16 mutant high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('26847029:publication', 'publication', {}),
                    ('22012066:publication', 'publication', {}),
                    ('25550361:publication', 'publication', {}),
                    ('22237022:publication', 'publication', {}),
                    ('24047651:publication', 'publication', {}),
                    ('Likely Loss-of-function:functional_effect', 'functional_effect', {}),
                    ('52:sample', 'sample', {}),
                    ('S1704Kfs*16:alteration', 'alteration', {'alteration_consequence': 'frameshift_variant', 'protein_start': '1703.0', 'protein_end': '1703.0'}),
                    ('77:patient', 'patient', {}),
                    ('13:variant', 'variant', {'timestamp': '2023-06-21 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'FGFR1 amplification results in overexpression of the protein. This mutation has been found in squamous cell lung, bladder and breast cancer (PMID: 22960745, 26451490, 24476821). In vitro studies demonstrate that FGFR1 overexpression in breast cancer cell lines promotes resistance to the ESR1 inhibitor fulvestrant as measured by sustained cell viability upon drug treatment (PMID: 31371343). A patient with FGFR1-amplified small cell lung cancer had a partial response to the FGFR-inhibitor pazopanib (Abstract: Russo et al. JCO PO, 2019. https://ascopubs.org/doi/full/10.1200/PO.19.00243).', 'variant_summary': 'FGFR1 amplification is known to be oncogenic.'}),
                    ('5:patient', 'patient', {}),
                    ('FGFR1:gene_hugo', 'gene_hugo', {'entrez_gene_id': '2260', 'gene_summary': 'FGFR1, a receptor tyrosine kinase, is altered by mutation, chromosomal rearrangement or amplification in various cancer types including lung and breast cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'While there is promising clinical data supporting the use of pan-FGFR-targeted inhibitors such as erdafitinib in patients with high-level FGFR1-amplified squamous cell lung cancer, their clinical utility in patients with FGFR1-amplified high-grade serous ovarian cancer is unknown.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('31371343:publication', 'publication', {}),
                    ('22960745:publication', 'publication', {}),
                    ('24476821:publication', 'publication', {}),
                    ('26451490:publication', 'publication', {}),
                    ('Erdafitinib:drug', 'drug', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('1:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('5:patient', 'patient', {}),
                    ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
                    ('LEVEL_3B:oncokb_evidence_level', 'oncokb_evidence_level', {}),
                    ('14:variant', 'variant', {'timestamp': '2019-08-28 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'Amplification of MYC is correlated with an increase in the expression of the MYC gene (PMID: 22464321,15083194), which leads to the transcriptional activation of MYC target genes. MYC target genes regulate various cellular processes involved in oncogenesis including proliferation, inhibiting exit from the cell cycle, stimulating vascularization and enhancing genomic instability (PMID: 22464321,10378696,16934487,19029958). Genomic amplification of MYC is observed in a large number of tumor types. A study of genomic amplification across many cancer types identified focal amplifications of MYC in 14% of all cancers, with additional enrichment for large scale arm-level copy number alterations encompassing the MYC gene locus (PMID: 20164920).', 'variant_summary': 'MYC amplification is known to be oncogenic.'}),
                    ('94:patient', 'patient', {}),
                    ('MYC:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4609', 'gene_summary': 'MYC, a transcription factor, is altered by chromosomal rearrangement, amplification and overexpression in a variety of cancer types.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with MYC-amplified high-grade serous ovarian cancer.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('20164920:publication', 'publication', {}),
                    ('16934487:publication', 'publication', {}),
                    ('10378696:publication', 'publication', {}),
                    ('15083194:publication', 'publication', {}),
                    ('19029958:publication', 'publication', {}),
                    ('22464321:publication', 'publication', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('22:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('94:patient', 'patient', {}),
                    ('15:variant', 'variant', {'timestamp': '2023-06-21 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'FGFR1 amplification results in overexpression of the protein. This mutation has been found in squamous cell lung, bladder and breast cancer (PMID: 22960745, 26451490, 24476821). In vitro studies demonstrate that FGFR1 overexpression in breast cancer cell lines promotes resistance to the ESR1 inhibitor fulvestrant as measured by sustained cell viability upon drug treatment (PMID: 31371343). A patient with FGFR1-amplified small cell lung cancer had a partial response to the FGFR-inhibitor pazopanib (Abstract: Russo et al. JCO PO, 2019. https://ascopubs.org/doi/full/10.1200/PO.19.00243).', 'variant_summary': 'FGFR1 amplification is known to be oncogenic.'}),
                    ('2:patient', 'patient', {}),
                    ('FGFR1:gene_hugo', 'gene_hugo', {'entrez_gene_id': '2260', 'gene_summary': 'FGFR1, a receptor tyrosine kinase, is altered by mutation, chromosomal rearrangement or amplification in various cancer types including lung and breast cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'While there is promising clinical data supporting the use of pan-FGFR-targeted inhibitors such as erdafitinib in patients with high-level FGFR1-amplified squamous cell lung cancer, their clinical utility in patients with FGFR1-amplified high-grade serous ovarian cancer is unknown.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('31371343:publication', 'publication', {}),
                    ('22960745:publication', 'publication', {}),
                    ('24476821:publication', 'publication', {}),
                    ('26451490:publication', 'publication', {}),
                    ('Erdafitinib:drug', 'drug', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('30:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('2:patient', 'patient', {}),
                    ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
                    ('LEVEL_3B:oncokb_evidence_level', 'oncokb_evidence_level', {}),
                    ('16:variant', 'variant', {'timestamp': '2019-08-28 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'Amplification of MYC is correlated with an increase in the expression of the MYC gene (PMID: 22464321,15083194), which leads to the transcriptional activation of MYC target genes. MYC target genes regulate various cellular processes involved in oncogenesis including proliferation, inhibiting exit from the cell cycle, stimulating vascularization and enhancing genomic instability (PMID: 22464321,10378696,16934487,19029958). Genomic amplification of MYC is observed in a large number of tumor types. A study of genomic amplification across many cancer types identified focal amplifications of MYC in 14% of all cancers, with additional enrichment for large scale arm-level copy number alterations encompassing the MYC gene locus (PMID: 20164920).', 'variant_summary': 'MYC amplification is known to be oncogenic.'}),
                    ('41:patient', 'patient', {}),
                    ('MYC:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4609', 'gene_summary': 'MYC, a transcription factor, is altered by chromosomal rearrangement, amplification and overexpression in a variety of cancer types.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with MYC-amplified high-grade serous ovarian cancer.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('20164920:publication', 'publication', {}),
                    ('16934487:publication', 'publication', {}),
                    ('10378696:publication', 'publication', {}),
                    ('15083194:publication', 'publication', {}),
                    ('19029958:publication', 'publication', {}),
                    ('22464321:publication', 'publication', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('85:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('41:patient', 'patient', {}),
                    ('17:variant', 'variant', {'timestamp': '2021-10-05 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'Chromosome 9p24.1 has been shown to include JAK2 as well as the programmed cell death-1 (PD1) ligand genes, PD-L1 and PD-L2; its amplification is often detected in Classical Hodgkin lymphoma (cHL) and mediastinal large B-cell lymphoma (MLBCL). Amplification of 9p24.1 was associated with increased protein expression and activity as well as increased PD-1 ligand transcription and enhanced sensitivity to JAK2 inhibition (PMID: 20628145).', 'variant_summary': 'JAK2 amplification is known to be oncogenic.'}),
                    ('40:patient', 'patient', {}),
                    ('JAK2:gene_hugo', 'gene_hugo', {'entrez_gene_id': '3717', 'gene_summary': 'JAK2, an intracellular kinase, is frequently altered by mutation or chromosomal rearrangement in hematologic malignancies.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with JAK2-amplified high-grade serous ovarian cancer.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('20628145:publication', 'publication', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('57:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('40:patient', 'patient', {}),
                    ('18:variant', 'variant', {'timestamp': '2019-01-16 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'Amplification of CD274 (more commonly PD-L1) has been detected across several tumor types including Hodgkin`s lymphoma (HL), mediastinal large B-cell lymphoma (MLBCL), small cell lung cancer, among others (PMID: 20628145, 29902298, 29377256, 27069084, 27620277). In preclinical assays, cells overexpressing PD-L1 exhibit increased features of immune evasion, due to binding of the PD-1 cell surface receptor expressed on immune cells (PMID: 28302645). In addition, the tyrosine kinase JAK2 is included in the 9p24.1 amplification region of PD-L1, which can also activate transcription of PD-L1 (PMID: 20628145). Preclinical studies have demonstrated that HL cells with PD-L1 amplification are sensitive to JAK2 inhibition (PMID: 24610827, 29467301). Patients with PD-L1 amplification have been found to have objective responses to PD-1/PD-L1-targeted immune therapies (PMID: 29902298, 28356247).', 'variant_summary': 'CD274 amplification is likely oncogenic.'}),
                    ('48:patient', 'patient', {}),
                    ('CD274:gene_hugo', 'gene_hugo', {'entrez_gene_id': '29126', 'gene_summary': 'CD274 (also known as PD-L1) is an immune receptor ligand. Expression of PD-L1 may help predict response to immunotherapies targeted against PD-L1 and its receptor, PD-1.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with CD274-amplified high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('28302645:publication', 'publication', {}),
                    ('29377256:publication', 'publication', {}),
                    ('20628145:publication', 'publication', {}),
                    ('29902298:publication', 'publication', {}),
                    ('27069084:publication', 'publication', {}),
                    ('28356247:publication', 'publication', {}),
                    ('24610827:publication', 'publication', {}),
                    ('29467301:publication', 'publication', {}),
                    ('27620277:publication', 'publication', {}),
                    ('Likely Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('52:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('48:patient', 'patient', {}),
                    ('19:variant', 'variant', {'timestamp': '2023-01-11 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'Amplification of PDCD1LG2 has been detected across several tumor types including Hodgkin`s lymphoma and mediastinal large B-cell lymphoma (PMID: 27069084, 29377256, 26850007, 26317899). In biochemical assays, cells overexpressing PDCD1LG2 demonstrate increased features of immune evasion from binding of the PD-1 cell surface receptor expressed on immune cells (PMID: 29167175). In addition, the tyrosine kinase JAK2 and PDCD1LG1 are included in the 9p24.1 amplification region of PDCD1LG2 (PMID: 20628145). Preclinical studies have demonstrated that cHL and MLBCL cell lines with PDCD1LG2 amplification are sensitive to JAK2 inhibition as measured by decreased tumor growth and prolonged survival (PMID: 24610827, 29467301). Patients with PDCD1LG2 amplification have been found to have an objective response to PD-1/PD-L1-targeted immune therapies (PMID: 29902298, 28356247).', 'variant_summary': 'PDCD1LG2 amplification is likely oncogenic.'}),
                    ('31:patient', 'patient', {}),
                    ('PDCD1LG2:gene_hugo', 'gene_hugo', {'entrez_gene_id': '80380', 'gene_summary': 'PDCD1LG2, a ligand of T-cell receptors involved in immune suppression, is overexpressed in various cancer types.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with PDCD1LG2-amplified high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('29377256:publication', 'publication', {}),
                    ('20628145:publication', 'publication', {}),
                    ('29902298:publication', 'publication', {}),
                    ('27069084:publication', 'publication', {}),
                    ('26850007:publication', 'publication', {}),
                    ('28356247:publication', 'publication', {}),
                    ('24610827:publication', 'publication', {}),
                    ('29467301:publication', 'publication', {}),
                    ('29167175:publication', 'publication', {}),
                    ('26317899:publication', 'publication', {}),
                    ('Likely Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('31:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('31:patient', 'patient', {}),
                    ('20:variant', 'variant', {'timestamp': '2019-05-31 00:00:00', 'version': 'v4.10', 'variant_summary': 'CDKN2A amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('40:patient', 'patient', {}),
                    ('CDKN2A:gene_hugo', 'gene_hugo', {'entrez_gene_id': '1029', 'gene_summary': 'The CDKN2A gene encodes two proteins, p16INK4A and p14ARF, that regulate the cell growth and survival. CDKN2A is altered by mutation and/or deletion in a broad range of solid and hematologic cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with CDKN2A-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('12:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('40:patient', 'patient', {}),
                    ('21:variant', 'variant', {'timestamp': '2019-04-18 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'PIK3CA amplification is an infrequent event across cancers (PMID: 28481359), although several individual studies have reported them in lung squamous cell carcinoma, ovarian, esophagus, cervical, and head and neck tumors (PMID: 22960745, 21720365, 25631445, 23636398). PIK3CA gene amplification has been correlated with PIK3CA protein overexpression (PMID: 17992665,19697359). PIK3CA overexpression enhances growth and migration (PMID: 17803655, 26876212). PIK3CA amplifications associate with poor prognosis in several cancers (PMID: 22994622, 22292935, 27095573). Some studies have found a correlation between PIK3CA copy number amplification and sensitivity to PI3K inhibitors (PMID: 24608574, 23136191). Amplification of PIK3CA known pathogenic mutant alleles have also been reported (PMID: 19706770). Expression of this mutation in a patient-derived xenograft model of lung squamous cell carcinoma demonstrated that it was not sensitive to the pan-PIK3CA inhibitor BKM120 similar to PDX models expressing wildtype PIK3CA (PMID: 30093452).', 'variant_summary': 'PIK3CA amplification is likely oncogenic.'}),
                    ('96:patient', 'patient', {}),
                    ('PIK3CA:gene_hugo', 'gene_hugo', {'entrez_gene_id': '5290', 'gene_summary': 'PIK3CA, the catalytic subunit of PI3-kinase, is frequently mutated in a diverse range of cancers including breast, endometrial and cervical cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with PIK3CA-amplified high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('21720365:publication', 'publication', {}),
                    ('22960745:publication', 'publication', {}),
                    ('17992665:publication', 'publication', {}),
                    ('23136191:publication', 'publication', {}),
                    ('30093452:publication', 'publication', {}),
                    ('19706770:publication', 'publication', {}),
                    ('28481359:publication', 'publication', {}),
                    ('22292935:publication', 'publication', {}),
                    ('22994622:publication', 'publication', {}),
                    ('19697359:publication', 'publication', {}),
                    ('25631445:publication', 'publication', {}),
                    ('24608574:publication', 'publication', {}),
                    ('26876212:publication', 'publication', {}),
                    ('23636398:publication', 'publication', {}),
                    ('27095573:publication', 'publication', {}),
                    ('17803655:publication', 'publication', {}),
                    ('Likely Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('84:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('96:patient', 'patient', {}),
                    ('22:variant', 'variant', {'timestamp': '2019-02-06 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'The amplification of BCL6 has been observed across a range of tumor types. Specifically, in follicular lymphoma BCL6 amplification was observed in a cohort of tumors lacking otherwise typical IGH/BCL2 rearrangements (PMID: 18500267, 24887414). BCL6 is a transcriptional regulator in germinal centers, from which human lymphomas often derive. In model systems, overexpression of BCL6 results in an anti-apoptotic and chemotherapy resistant phenotype (PMID: 15577913, 16142238).', 'variant_summary': 'BCL6 amplification is likely oncogenic.'}),
                    ('55:patient', 'patient', {}),
                    ('BCL6:gene_hugo', 'gene_hugo', {'entrez_gene_id': '604', 'gene_summary': 'BCL6, a transcriptional repressor involved in immune cell development, is frequently altered by chromosomal rearrangement in lymphomas.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with BCL6-amplified high-grade serous ovarian cancer.'}),
                    ('Likely Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('16142238:publication', 'publication', {}),
                    ('18500267:publication', 'publication', {}),
                    ('24887414:publication', 'publication', {}),
                    ('15577913:publication', 'publication', {}),
                    ('Likely Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('35:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('55:patient', 'patient', {}),
                    ('23:variant', 'variant', {'timestamp': '2019-02-06 00:00:00', 'version': 'v4.10', 'variant_summary': 'TP63 amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('82:patient', 'patient', {}),
                    ('TP63:gene_hugo', 'gene_hugo', {'entrez_gene_id': '8626', 'gene_summary': 'TP63, a transcription factor, is recurrently altered by chromosomal rearrangement in hematologic malignancies.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with TP63-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('23:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('82:patient', 'patient', {}),
                    ('24:variant', 'variant', {'timestamp': '2019-07-30 00:00:00', 'version': 'v4.10', 'variant_summary': 'IL7R amplification has not specifically been reviewed by the OncoKB team, and therefore its biological significance is unknown.'}),
                    ('54:patient', 'patient', {}),
                    ('IL7R:gene_hugo', 'gene_hugo', {'entrez_gene_id': '3575', 'gene_summary': 'IL7R, a subunit of the interleukin 7 receptor, is altered in various hematologic malignancies including pediatric acute lymphoblastic leukemia.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'There are no FDA-approved or NCCN-compendium listed treatments specifically for patients with IL7R-amplified high-grade serous ovarian cancer.'}),
                    ('Unknown:oncogenicity', 'oncogenicity', {}),
                    ('Unknown:functional_effect', 'functional_effect', {}),
                    ('67:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('54:patient', 'patient', {}),
                    ('25:variant', 'variant', {'timestamp': '2023-06-21 00:00:00', 'version': 'v4.10', 'mutation_effect_description': 'FGFR1 amplification results in overexpression of the protein. This mutation has been found in squamous cell lung, bladder and breast cancer (PMID: 22960745, 26451490, 24476821). In vitro studies demonstrate that FGFR1 overexpression in breast cancer cell lines promotes resistance to the ESR1 inhibitor fulvestrant as measured by sustained cell viability upon drug treatment (PMID: 31371343). A patient with FGFR1-amplified small cell lung cancer had a partial response to the FGFR-inhibitor pazopanib (Abstract: Russo et al. JCO PO, 2019. https://ascopubs.org/doi/full/10.1200/PO.19.00243).', 'variant_summary': 'FGFR1 amplification is known to be oncogenic.'}),
                    ('16:patient', 'patient', {}),
                    ('FGFR1:gene_hugo', 'gene_hugo', {'entrez_gene_id': '2260', 'gene_summary': 'FGFR1, a receptor tyrosine kinase, is altered by mutation, chromosomal rearrangement or amplification in various cancer types including lung and breast cancers.'}),
                    ('HGSOC:disease', 'disease', {'tumor_type_summary': 'While there is promising clinical data supporting the use of pan-FGFR-targeted inhibitors such as erdafitinib in patients with high-level FGFR1-amplified squamous cell lung cancer, their clinical utility in patients with FGFR1-amplified high-grade serous ovarian cancer is unknown.'}),
                    ('Oncogenic:oncogenicity', 'oncogenicity', {}),
                    ('31371343:publication', 'publication', {}),
                    ('22960745:publication', 'publication', {}),
                    ('24476821:publication', 'publication', {}),
                    ('26451490:publication', 'publication', {}),
                    ('Erdafitinib:drug', 'drug', {}),
                    ('Gain-of-function:functional_effect', 'functional_effect', {}),
                    ('68:sample', 'sample', {}),
                    ('Amplification:alteration', 'alteration', {}),
                    ('16:patient', 'patient', {}),
                    ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
                    ('LEVEL_3B:oncokb_evidence_level', 'oncokb_evidence_level', {}),
                    ]

    assert_edges = [('', '0:variant', '33:patient', 'patient_has_variant', {}),
                    ('', '0:variant', 'MET:gene_hugo', 'variant_in_gene', {}),
                    ('', '0:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '0:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '0:variant', '30073261:publication', 'published', {}),
                    ('', '0:variant', '16461907:publication', 'published', {}),
                    ('', '0:variant', '19117057:publication', 'published', {}),
                    ('', '0:variant', '17667909:publication', 'published', {}),
                    ('', '0:variant', '22869872:publication', 'published', {}),
                    ('', '0:variant', '18077425:publication', 'published', {}),
                    ('', '0:variant', '18093943:publication', 'published', {}),
                    ('', '0:variant', '28481359:publication', 'published', {}),
                    ('', '0:variant', '17463250:publication', 'published', {}),
                    ('', '0:variant', 'Crizotinib:drug', 'variant_affected_by_drug', {}),
                    ('', '0:variant', 'Telisotuzumab Vedotin:drug', 'variant_affected_by_drug', {}),
                    ('', '0:variant', 'Capmatinib:drug', 'variant_affected_by_drug', {}),
                    ('', '0:variant', 'Tepotinib:drug', 'variant_affected_by_drug', {}),
                    ('', '0:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '0:variant', '91:sample', 'variant_in_sample', {}),
                    ('', '0:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '91:sample', '33:patient', 'sample_to_patient', {}),
                    ('', '0:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
                    ('', '0:variant', 'LEVEL_3B:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
                    ('', '1:variant', '67:patient', 'patient_has_variant', {}),
                    ('', '1:variant', 'BRAF:gene_hugo', 'variant_in_gene', {}),
                    ('', '1:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '1:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '1:variant', '24055054:publication', 'published', {}),
                    ('', '1:variant', '25618114:publication', 'published', {}),
                    ('', '1:variant', '22395615:publication', 'published', {}),
                    ('', '1:variant', '21098728:publication', 'published', {}),
                    ('', '1:variant', '15467732:publication', 'published', {}),
                    ('', '1:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '1:variant', '4:sample', 'variant_in_sample', {}),
                    ('', '1:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '4:sample', '67:patient', 'sample_to_patient', {}),
                    ('', '2:variant', '88:patient', 'patient_has_variant', {}),
                    ('', '2:variant', 'EZH2:gene_hugo', 'variant_in_gene', {}),
                    ('', '2:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '2:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '2:variant', '16330673:publication', 'published', {}),
                    ('', '2:variant', '12374981:publication', 'published', {}),
                    ('', '2:variant', '23529930:publication', 'published', {}),
                    ('', '2:variant', '14500907:publication', 'published', {}),
                    ('', '2:variant', '21837672:publication', 'published', {}),
                    ('', '2:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '2:variant', '92:sample', 'variant_in_sample', {}),
                    ('', '2:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '92:sample', '88:patient', 'sample_to_patient', {}),
                    ('', '3:variant', '60:patient', 'patient_has_variant', {}),
                    ('', '3:variant', 'CDKN2A:gene_hugo', 'variant_in_gene', {}),
                    ('', '3:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '3:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '3:variant', '21526190:publication', 'published', {}),
                    ('', '3:variant', '28982163:publication', 'published', {}),
                    ('', '3:variant', '26516359:publication', 'published', {}),
                    ('', '3:variant', '15495191:publication', 'published', {}),
                    ('', '3:variant', '10999737:publication', 'published', {}),
                    ('', '3:variant', 'Palbociclib:drug', 'variant_affected_by_drug', {}),
                    ('', '3:variant', 'Ribociclib:drug', 'variant_affected_by_drug', {}),
                    ('', '3:variant', 'Abemaciclib:drug', 'variant_affected_by_drug', {}),
                    ('', '3:variant', 'Loss-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '3:variant', '37:sample', 'variant_in_sample', {}),
                    ('', '3:variant', 'Deletion:alteration', 'variant_to_alteration', {}),
                    ('', '37:sample', '60:patient', 'sample_to_patient', {}),
                    ('', '3:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
                    ('', '3:variant', 'LEVEL_4:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
                    ('', '4:variant', '32:patient', 'patient_has_variant', {}),
                    ('', '4:variant', 'ETV6:gene_hugo', 'variant_in_gene', {}),
                    ('', '4:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '4:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '4:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '4:variant', '15:sample', 'variant_in_sample', {}),
                    ('', '4:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '15:sample', '32:patient', 'sample_to_patient', {}),
                    ('', '5:variant', '40:patient', 'patient_has_variant', {}),
                    ('', '5:variant', 'ETNK1:gene_hugo', 'variant_in_gene', {}),
                    ('', '5:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '5:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '5:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '5:variant', '43:sample', 'variant_in_sample', {}),
                    ('', '5:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '43:sample', '40:patient', 'sample_to_patient', {}),
                    ('', '6:variant', '68:patient', 'patient_has_variant', {}),
                    ('', '6:variant', 'KRAS:gene_hugo', 'variant_in_gene', {}),
                    ('', '6:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '6:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '6:variant', '23099803:publication', 'published', {}),
                    ('', '6:variant', '29808010:publication', 'published', {}),
                    ('', '6:variant', '30072474:publication', 'published', {}),
                    ('', '6:variant', '20841479:publication', 'published', {}),
                    ('', '6:variant', '23404247:publication', 'published', {}),
                    ('', '6:variant', 'Likely Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '6:variant', '76:sample', 'variant_in_sample', {}),
                    ('', '6:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '76:sample', '68:patient', 'sample_to_patient', {}),
                    ('', '7:variant', '65:patient', 'patient_has_variant', {}),
                    ('', '7:variant', 'NTRK3:gene_hugo', 'variant_in_gene', {}),
                    ('', '7:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '7:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '7:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '7:variant', '46:sample', 'variant_in_sample', {}),
                    ('', '7:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '46:sample', '65:patient', 'sample_to_patient', {}),
                    ('', '8:variant', '35:patient', 'patient_has_variant', {}),
                    ('', '8:variant', 'IDH2:gene_hugo', 'variant_in_gene', {}),
                    ('', '8:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '8:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '8:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '8:variant', '13:sample', 'variant_in_sample', {}),
                    ('', '8:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '13:sample', '35:patient', 'sample_to_patient', {}),
                    ('', '9:variant', '75:patient', 'patient_has_variant', {}),
                    ('', '9:variant', 'MAF:gene_hugo', 'variant_in_gene', {}),
                    ('', '9:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '9:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '9:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '9:variant', '85:sample', 'variant_in_sample', {}),
                    ('', '9:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '85:sample', '75:patient', 'sample_to_patient', {}),
                    ('', '10:variant', '61:patient', 'patient_has_variant', {}),
                    ('', '10:variant', 'BRCA1:gene_hugo', 'variant_in_gene', {}),
                    ('', '10:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '10:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '10:variant', '11358863:publication', 'published', {}),
                    ('', '10:variant', '20608970:publication', 'published', {}),
                    ('', '10:variant', '12483515:publication', 'published', {}),
                    ('', '10:variant', '12947386:publication', 'published', {}),
                    ('', '10:variant', 'Olaparib:drug', 'variant_affected_by_drug', {}),
                    ('', '10:variant', 'Rucaparib:drug', 'variant_affected_by_drug', {}),
                    ('', '10:variant', 'Niraparib:drug', 'variant_affected_by_drug', {}),
                    ('', '10:variant', 'Talazoparib:drug', 'variant_affected_by_drug', {}),
                    ('', '10:variant', 'Likely Loss-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '10:variant', '8:sample', 'variant_in_sample', {}),
                    ('', '10:variant', 'Y1463*:alteration', 'variant_to_alteration', {}),
                    ('', '8:sample', '61:patient', 'sample_to_patient', {}),
                    ('', '10:variant', 'LEVEL_Fda2:fda_evidence_level', 'variant_to_fda_evidence', {}),
                    ('', '10:variant', 'LEVEL_1:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
                    ('', '11:variant', '11:patient', 'patient_has_variant', {}),
                    ('', '11:variant', 'TP53:gene_hugo', 'variant_in_gene', {}),
                    ('', '11:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '11:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '11:variant', '19336573:publication', 'published', {}),
                    ('', '11:variant', '27759562:publication', 'published', {}),
                    ('', '11:variant', '16007150:publication', 'published', {}),
                    ('', '11:variant', '11753428:publication', 'published', {}),
                    ('', '11:variant', '11900253:publication', 'published', {}),
                    ('', '11:variant', '21467160:publication', 'published', {}),
                    ('', '11:variant', 'Likely Loss-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '11:variant', '1:sample', 'variant_in_sample', {}),
                    ('', '11:variant', 'X125_splice:alteration', 'variant_to_alteration', {}),
                    ('', '1:sample', '11:patient', 'sample_to_patient', {}),
                    ('', '12:variant', '77:patient', 'patient_has_variant', {}),
                    ('', '12:variant', 'BCOR:gene_hugo', 'variant_in_gene', {}),
                    ('', '12:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '12:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '12:variant', '26847029:publication', 'published', {}),
                    ('', '12:variant', '22012066:publication', 'published', {}),
                    ('', '12:variant', '25550361:publication', 'published', {}),
                    ('', '12:variant', '22237022:publication', 'published', {}),
                    ('', '12:variant', '24047651:publication', 'published', {}),
                    ('', '12:variant', 'Likely Loss-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '12:variant', '52:sample', 'variant_in_sample', {}),
                    ('', '12:variant', 'S1704Kfs*16:alteration', 'variant_to_alteration', {}),
                    ('', '52:sample', '77:patient', 'sample_to_patient', {}),
                    ('', '13:variant', '5:patient', 'patient_has_variant', {}),
                    ('', '13:variant', 'FGFR1:gene_hugo', 'variant_in_gene', {}),
                    ('', '13:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '13:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '13:variant', '31371343:publication', 'published', {}),
                    ('', '13:variant', '22960745:publication', 'published', {}),
                    ('', '13:variant', '24476821:publication', 'published', {}),
                    ('', '13:variant', '26451490:publication', 'published', {}),
                    ('', '13:variant', 'Erdafitinib:drug', 'variant_affected_by_drug', {}),
                    ('', '13:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '13:variant', '1:sample', 'variant_in_sample', {}),
                    ('', '13:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '1:sample', '5:patient', 'sample_to_patient', {}),
                    ('', '13:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
                    ('', '13:variant', 'LEVEL_3B:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
                    ('', '14:variant', '94:patient', 'patient_has_variant', {}),
                    ('', '14:variant', 'MYC:gene_hugo', 'variant_in_gene', {}),
                    ('', '14:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '14:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '14:variant', '20164920:publication', 'published', {}),
                    ('', '14:variant', '16934487:publication', 'published', {}),
                    ('', '14:variant', '10378696:publication', 'published', {}),
                    ('', '14:variant', '15083194:publication', 'published', {}),
                    ('', '14:variant', '19029958:publication', 'published', {}),
                    ('', '14:variant', '22464321:publication', 'published', {}),
                    ('', '14:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '14:variant', '22:sample', 'variant_in_sample', {}),
                    ('', '14:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '22:sample', '94:patient', 'sample_to_patient', {}),
                    ('', '15:variant', '2:patient', 'patient_has_variant', {}),
                    ('', '15:variant', 'FGFR1:gene_hugo', 'variant_in_gene', {}),
                    ('', '15:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '15:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '15:variant', '31371343:publication', 'published', {}),
                    ('', '15:variant', '22960745:publication', 'published', {}),
                    ('', '15:variant', '24476821:publication', 'published', {}),
                    ('', '15:variant', '26451490:publication', 'published', {}),
                    ('', '15:variant', 'Erdafitinib:drug', 'variant_affected_by_drug', {}),
                    ('', '15:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '15:variant', '30:sample', 'variant_in_sample', {}),
                    ('', '15:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '30:sample', '2:patient', 'sample_to_patient', {}),
                    ('', '15:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
                    ('', '15:variant', 'LEVEL_3B:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
                    ('', '16:variant', '41:patient', 'patient_has_variant', {}),
                    ('', '16:variant', 'MYC:gene_hugo', 'variant_in_gene', {}),
                    ('', '16:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '16:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '16:variant', '20164920:publication', 'published', {}),
                    ('', '16:variant', '16934487:publication', 'published', {}),
                    ('', '16:variant', '10378696:publication', 'published', {}),
                    ('', '16:variant', '15083194:publication', 'published', {}),
                    ('', '16:variant', '19029958:publication', 'published', {}),
                    ('', '16:variant', '22464321:publication', 'published', {}),
                    ('', '16:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '16:variant', '85:sample', 'variant_in_sample', {}),
                    ('', '16:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '85:sample', '41:patient', 'sample_to_patient', {}),
                    ('', '17:variant', '40:patient', 'patient_has_variant', {}),
                    ('', '17:variant', 'JAK2:gene_hugo', 'variant_in_gene', {}),
                    ('', '17:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '17:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '17:variant', '20628145:publication', 'published', {}),
                    ('', '17:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '17:variant', '57:sample', 'variant_in_sample', {}),
                    ('', '17:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '57:sample', '40:patient', 'sample_to_patient', {}),
                    ('', '18:variant', '48:patient', 'patient_has_variant', {}),
                    ('', '18:variant', 'CD274:gene_hugo', 'variant_in_gene', {}),
                    ('', '18:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '18:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '18:variant', '28302645:publication', 'published', {}),
                    ('', '18:variant', '29377256:publication', 'published', {}),
                    ('', '18:variant', '20628145:publication', 'published', {}),
                    ('', '18:variant', '29902298:publication', 'published', {}),
                    ('', '18:variant', '27069084:publication', 'published', {}),
                    ('', '18:variant', '28356247:publication', 'published', {}),
                    ('', '18:variant', '24610827:publication', 'published', {}),
                    ('', '18:variant', '29467301:publication', 'published', {}),
                    ('', '18:variant', '27620277:publication', 'published', {}),
                    ('', '18:variant', 'Likely Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '18:variant', '52:sample', 'variant_in_sample', {}),
                    ('', '18:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '52:sample', '48:patient', 'sample_to_patient', {}),
                    ('', '19:variant', '31:patient', 'patient_has_variant', {}),
                    ('', '19:variant', 'PDCD1LG2:gene_hugo', 'variant_in_gene', {}),
                    ('', '19:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '19:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '19:variant', '29377256:publication', 'published', {}),
                    ('', '19:variant', '20628145:publication', 'published', {}),
                    ('', '19:variant', '29902298:publication', 'published', {}),
                    ('', '19:variant', '27069084:publication', 'published', {}),
                    ('', '19:variant', '26850007:publication', 'published', {}),
                    ('', '19:variant', '28356247:publication', 'published', {}),
                    ('', '19:variant', '24610827:publication', 'published', {}),
                    ('', '19:variant', '29467301:publication', 'published', {}),
                    ('', '19:variant', '29167175:publication', 'published', {}),
                    ('', '19:variant', '26317899:publication', 'published', {}),
                    ('', '19:variant', 'Likely Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '19:variant', '31:sample', 'variant_in_sample', {}),
                    ('', '19:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '31:sample', '31:patient', 'sample_to_patient', {}),
                    ('', '20:variant', '40:patient', 'patient_has_variant', {}),
                    ('', '20:variant', 'CDKN2A:gene_hugo', 'variant_in_gene', {}),
                    ('', '20:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '20:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '20:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '20:variant', '12:sample', 'variant_in_sample', {}),
                    ('', '20:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '12:sample', '40:patient', 'sample_to_patient', {}),
                    ('', '21:variant', '96:patient', 'patient_has_variant', {}),
                    ('', '21:variant', 'PIK3CA:gene_hugo', 'variant_in_gene', {}),
                    ('', '21:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '21:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '21:variant', '21720365:publication', 'published', {}),
                    ('', '21:variant', '22960745:publication', 'published', {}),
                    ('', '21:variant', '17992665:publication', 'published', {}),
                    ('', '21:variant', '23136191:publication', 'published', {}),
                    ('', '21:variant', '30093452:publication', 'published', {}),
                    ('', '21:variant', '19706770:publication', 'published', {}),
                    ('', '21:variant', '28481359:publication', 'published', {}),
                    ('', '21:variant', '22292935:publication', 'published', {}),
                    ('', '21:variant', '22994622:publication', 'published', {}),
                    ('', '21:variant', '19697359:publication', 'published', {}),
                    ('', '21:variant', '25631445:publication', 'published', {}),
                    ('', '21:variant', '24608574:publication', 'published', {}),
                    ('', '21:variant', '26876212:publication', 'published', {}),
                    ('', '21:variant', '23636398:publication', 'published', {}),
                    ('', '21:variant', '27095573:publication', 'published', {}),
                    ('', '21:variant', '17803655:publication', 'published', {}),
                    ('', '21:variant', 'Likely Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '21:variant', '84:sample', 'variant_in_sample', {}),
                    ('', '21:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '84:sample', '96:patient', 'sample_to_patient', {}),
                    ('', '22:variant', '55:patient', 'patient_has_variant', {}),
                    ('', '22:variant', 'BCL6:gene_hugo', 'variant_in_gene', {}),
                    ('', '22:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '22:variant', 'Likely Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '22:variant', '16142238:publication', 'published', {}),
                    ('', '22:variant', '18500267:publication', 'published', {}),
                    ('', '22:variant', '24887414:publication', 'published', {}),
                    ('', '22:variant', '15577913:publication', 'published', {}),
                    ('', '22:variant', 'Likely Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '22:variant', '35:sample', 'variant_in_sample', {}),
                    ('', '22:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '35:sample', '55:patient', 'sample_to_patient', {}),
                    ('', '23:variant', '82:patient', 'patient_has_variant', {}),
                    ('', '23:variant', 'TP63:gene_hugo', 'variant_in_gene', {}),
                    ('', '23:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '23:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '23:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '23:variant', '23:sample', 'variant_in_sample', {}),
                    ('', '23:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '23:sample', '82:patient', 'sample_to_patient', {}),
                    ('', '24:variant', '54:patient', 'patient_has_variant', {}),
                    ('', '24:variant', 'IL7R:gene_hugo', 'variant_in_gene', {}),
                    ('', '24:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '24:variant', 'Unknown:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '24:variant', 'Unknown:functional_effect', 'variant_has_effect', {}),
                    ('', '24:variant', '67:sample', 'variant_in_sample', {}),
                    ('', '24:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '67:sample', '54:patient', 'sample_to_patient', {}),
                    ('', '25:variant', '16:patient', 'patient_has_variant', {}),
                    ('', '25:variant', 'FGFR1:gene_hugo', 'variant_in_gene', {}),
                    ('', '25:variant', 'HGSOC:disease', 'variant_to_disease', {}),
                    ('', '25:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
                    ('', '25:variant', '31371343:publication', 'published', {}),
                    ('', '25:variant', '22960745:publication', 'published', {}),
                    ('', '25:variant', '24476821:publication', 'published', {}),
                    ('', '25:variant', '26451490:publication', 'published', {}),
                    ('', '25:variant', 'Erdafitinib:drug', 'variant_affected_by_drug', {}),
                    ('', '25:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
                    ('', '25:variant', '68:sample', 'variant_in_sample', {}),
                    ('', '25:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
                    ('', '68:sample', '16:patient', 'sample_to_patient', {}),
                    ('', '25:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
                    ('', '25:variant', 'LEVEL_3B:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
                    ]

    data_mapping = {f"tests/{directory_name}/data.csv": f"tests/{directory_name}/mapping.yaml"}

    nodes, edges = ontoweaver.extract(data_mapping, affix="suffix", parallel_mapping=8, validate_output=True, raise_errors=False)

    assert nodes, "There is no node"
    assert edges, "There is no edge"

    bc_nodes = ontoweaver.ow2bc(nodes)
    bc_edges = ontoweaver.ow2bc(edges)
    # The fusion functions is not being used due to the large number of duplicates, resulting in the properties being
    # fused in varying orders. This is not normally an issue, as the properties are still the same.
    # fnodes, fedges = ontoweaver.fusion.reconciliate(nodes, edges, reconciliate_sep=",")

    assert_node_set = testing_functions.convert_to_set(assert_nodes)
    f_node_set = testing_functions.convert_to_set(bc_nodes)

    assert assert_node_set == f_node_set, "Nodes are not equal."

    assert_edge_set = testing_functions.convert_to_set(assert_edges)
    f_edge_set = testing_functions.convert_to_set(bc_edges)

    assert assert_edge_set == f_edge_set, "Edges are not equal."

if __name__ == "__main__":
    test_oncokb()



def test_oncokb():
    from . import testing_functions
    import logging
    import ontoweaver

    logging.basicConfig(level=logging.DEBUG)

    directory_name = "oncokb"

    expected_nodes = [
        ('0:variant', 'variant', {'timestamp': '91', 'version': [], 'mutation_effect_description': 'MET amplification results from the gain of the MET gene on chromosome 7 (PMID: 17463250).', 'variant_summary': []}),
        ('33:patient', 'patient', {}),
        ('MET:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4233', 'gene_summary': 'MET, a receptor tyrosine kinase.'}),
        ('HGSOC:disease', 'disease', {'tumor_type_summary': []}),
        ('Oncogenic:oncogenicity', 'oncogenicity', {}),
        ('30073261:publication', 'publication', {}),
        ('16461907:publication', 'publication', {}),
        ('19117057:publication', 'publication', {}),
        ('17667909:publication', 'publication', {}),
        ('22869872:publication', 'publication', {}),
        ('18077425:publication', 'publication', {}),
        ('18093943:publication', 'publication', {}),
        ('28481359:publication', 'publication', {}),
        ('17463250:publication', 'publication', {}),
        ('2023-02-26:drug', 'drug', {}),
        ('Gain-of-function:functional_effect', 'functional_effect', {}),
        ('Amplification:alteration', 'alteration', {'alteration_consequence': [], 'protein_start': [], 'protein_end': []}),
        ('33:patient', 'patient', {}),
        ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
        ('LEVEL_3B:oncokb_evidence_level', 'oncokb_evidence_level', {}),
        ('1:variant', 'variant', {'timestamp': '92', 'version': [], 'mutation_effect_description': 'MET amplification results from the gain of the MET gene on chromosome 7 (PMID: 17463250).', 'variant_summary': []}),
        ('34:patient', 'patient', {}),
        ('MET2:gene_hugo', 'gene_hugo', {'entrez_gene_id': '4234', 'gene_summary': 'MET, a receptor tyrosine kinase.'}),
        ('HGSOC:disease', 'disease', {'tumor_type_summary': []}),
        ('Oncogenic:oncogenicity', 'oncogenicity', {}),
        ('30073261:publication', 'publication', {}),
        ('16461907:publication', 'publication', {}),
        ('2023-02-26:drug', 'drug', {}),
        ('Gain-of-function:functional_effect', 'functional_effect', {}),
        ('Amplification:alteration', 'alteration', {'alteration_consequence': [], 'protein_start': [], 'protein_end': []}),
        ('34:patient', 'patient', {}),
        ('LEVEL_Fda3:fda_evidence_level', 'fda_evidence_level', {}),
        ('LEVEL_3B:oncokb_evidence_level', 'oncokb_evidence_level', {})
      ]

    expected_edges = [
        ('(0:variant)--[patient_has_variant]->(33:patient)', '0:variant', '33:patient', 'patient_has_variant', {}),
        ('(0:variant)--[variant_in_gene]->(MET:gene_hugo)', '0:variant', 'MET:gene_hugo', 'variant_in_gene', {}),
        ('(0:variant)--[variant_to_disease]->(HGSOC:disease)', '0:variant', 'HGSOC:disease', 'variant_to_disease', {}),
        ('(0:variant)--[variant_to_oncogenicity]->(Oncogenic:oncogenicity)', '0:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
        ('(0:variant)--[published]->(30073261:publication)', '0:variant', '30073261:publication', 'published', {}),
        ('(0:variant)--[published]->(16461907:publication)', '0:variant', '16461907:publication', 'published', {}),
        ('(0:variant)--[published]->(19117057:publication)', '0:variant', '19117057:publication', 'published', {}),
        ('(0:variant)--[published]->(17667909:publication)', '0:variant', '17667909:publication', 'published', {}),
        ('(0:variant)--[published]->(22869872:publication)', '0:variant', '22869872:publication', 'published', {}),
        ('(0:variant)--[published]->(18077425:publication)', '0:variant', '18077425:publication', 'published', {}),
        ('(0:variant)--[published]->(18093943:publication)', '0:variant', '18093943:publication', 'published', {}),
        ('(0:variant)--[published]->(28481359:publication)', '0:variant', '28481359:publication', 'published', {}),
        ('(0:variant)--[published]->(17463250:publication)', '0:variant', '17463250:publication', 'published', {}),
        ('(0:variant)--[variant_affected_by_drug]->(2023-02-26:drug)', '0:variant', '2023-02-26:drug', 'variant_affected_by_drug', {}),
        ('(0:variant)--[variant_has_effect]->(Gain-of-function:functional_effect)', '0:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
        ('(0:variant)--[variant_to_alteration]->(Amplification:alteration)', '0:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
        ('(0:variant)--[variant_to_fda_evidence]->(LEVEL_Fda3:fda_evidence_level)', '0:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
        ('(0:variant)--[variant_to_oncokb_evidence]->(LEVEL_3B:oncokb_evidence_level)', '0:variant', 'LEVEL_3B:oncokb_evidence_level', 'variant_to_oncokb_evidence', {}),
        ('(1:variant)--[patient_has_variant]->(34:patient)', '1:variant', '34:patient', 'patient_has_variant', {}),
        ('(1:variant)--[variant_in_gene]->(MET2:gene_hugo)', '1:variant', 'MET2:gene_hugo', 'variant_in_gene', {}),
        ('(1:variant)--[variant_to_disease]->(HGSOC:disease)', '1:variant', 'HGSOC:disease', 'variant_to_disease', {}),
        ('(1:variant)--[variant_to_oncogenicity]->(Oncogenic:oncogenicity)', '1:variant', 'Oncogenic:oncogenicity', 'variant_to_oncogenicity', {}),
        ('(1:variant)--[published]->(30073261:publication)', '1:variant', '30073261:publication', 'published', {}),
        ('(1:variant)--[published]->(16461907:publication)', '1:variant', '16461907:publication', 'published', {}),
        ('(1:variant)--[variant_affected_by_drug]->(2023-02-26:drug)', '1:variant', '2023-02-26:drug', 'variant_affected_by_drug', {}),
        ('(1:variant)--[variant_has_effect]->(Gain-of-function:functional_effect)', '1:variant', 'Gain-of-function:functional_effect', 'variant_has_effect', {}),
        ('(1:variant)--[variant_to_alteration]->(Amplification:alteration)', '1:variant', 'Amplification:alteration', 'variant_to_alteration', {}),
        ('(1:variant)--[variant_to_fda_evidence]->(LEVEL_Fda3:fda_evidence_level)', '1:variant', 'LEVEL_Fda3:fda_evidence_level', 'variant_to_fda_evidence', {}),
        ('(1:variant)--[variant_to_oncokb_evidence]->(LEVEL_3B:oncokb_evidence_level)', '1:variant', 'LEVEL_3B:oncokb_evidence_level', 'variant_to_oncokb_evidence', {})
    ]

    data_mapping = {f"tests/{directory_name}/data.csv": f"tests/{directory_name}/mapping.yaml"}

    nodes, edges = ontoweaver.extract(data_mapping, affix="suffix", validate_output=True, raise_errors=False)

    bc_nodes = ontoweaver.ow2bc(nodes)
    bc_edges = ontoweaver.ow2bc(edges)

    # logging.debug(bc_nodes)
    # logging.debug(bc_edges)
    testing_functions.assert_equals(bc_nodes, expected_nodes)
    testing_functions.assert_equals(bc_edges, expected_edges)


if __name__ == "__main__":
    test_oncokb()



import os
import glob
import pandas as pd

def get_csv_files(directory):
    """Get all CSV files in the directory."""
    return glob.glob(os.path.join(directory, "*.csv"))


def compare_csv_files(expected_dir, output_dir):
    """Compare all CSV files between two directories."""
    expected_files = get_csv_files(expected_dir)
    output_files = get_csv_files(output_dir)

    assert len(expected_files) == len(output_files), "The number of CSV files does not match."

    for expected_file in expected_files:
        filename = os.path.basename(expected_file)
        output_file = os.path.join(output_dir, filename)
        assert os.path.exists(output_file), f"Output file {filename} does not exist."

        expected_df = pd.read_csv(expected_file, on_bad_lines='skip')
        output_df = pd.read_csv(output_file, on_bad_lines='skip')

        pd.testing.assert_frame_equal(output_df, expected_df)


def convert_to_set(tuple_output):
    """Convert the OntoWeaver tuple output to a set."""

    out = set()
    for elem in tuple_output:
        id = elem[0]
        type = elem[1]
        if len(elem) == 3: # Nodes
            sp = {}
            for k,v in elem[2].items():
                if isinstance(v, list):
                    sp[k] = ",".join(v)
                else:
                    sp[k] = v
            props = tuple(sorted(sp.items()))
            out.add(tuple([id, type, props]))

        elif len(elem) == 4: # Edges
            source = elem[2]
            target = elem[3]
            props = tuple(sorted(elem[4].items()))
            out.add(tuple([id, type, source, target, props]))

    return out


def assert_equals(given, expected):
    given_set = convert_to_set(given)
    expected_set = convert_to_set(expected)
    assert len(expected_set) == len(given_set)
    for elem in given_set:
        assert elem in expected_set, f"Element {elem} should exists."


def create_parquet_files():
    df1 = pd.DataFrame({"a":[1,2], "b":[3,4]})
    df1.to_parquet("./tests/parquets/numbers_part-1.parquet")

    df2 = pd.DataFrame({"a":[5,6], "b":[7,8]})
    df2.to_parquet("./tests/parquets/numbers_part-2.parquet")




#!/usr/bin/env -S uv run --script
# /// script
# dependencies = [
#    "biocypher<1.0.0,>=0.11.0",
#    "pooch<2.0.0,>=1.7.0",
#    "pandas<3.0.0,>=2.3.1",
#    "numpy<3.0.0,>=2.2.4",
#    "owlready2<1.0,>=0.49",
#    "jsonargparse<5.0,>=4.39",
#    "xdg-base-dirs<7.0.0,>=6.0.2",
#    "pandera[io]<1.0.0,>=0.27.0",
#    "alive-progress<4.0,>=3.2",
#    "fsspec<2026.0.0,>=2025.10.0",
#    "natsort>5.0.0",
#    "lxml>6.0.0",
#    "jmespath>=1.0.1",
# ]
# ///

import re
import json

import rdflib
from rdflib import URIRef, Literal, Namespace
from rdflib.namespace import RDF, RDFS, OWL
from urllib.parse import quote_plus as url_quote

import logging
logger = logging.getLogger("biocypher_to_owl")


class default:
    root_name = "BioCypherRoot"
    remove_affix = "none"
    affix_sep = ":"


def clean_affix_uri(name, remove_affix = default.remove_affix, affix_sep = default.affix_sep):
    affix_sep = url_quote(affix_sep)

    if remove_affix != default.remove_affix:
        if remove_affix == "prefix":
            matched = re.search(r"(.+)#\w+" +affix_sep+ r"([\w%]*)$", name)
            if matched:
                assert len(matched.groups()) == 2
                clean = matched.groups()[0] + matched.groups()[1]
        elif remove_affix == "suffix":
            matched = re.search(r"(.+#[\w%]*)" +affix_sep+ r"\w+$", name)
            if matched:
                assert len(matched.groups()) == 1
                clean = matched.groups()[0]
    if matched:
        return clean
    else:
        return None


def clean_affix_literal(name, remove_affix = default.remove_affix, affix_sep = default.affix_sep):
    if remove_affix != default.remove_affix:
        if remove_affix == "prefix":
            matched = re.search(r"^\w+" +affix_sep+ r"([\w%]*)$", name)
            if matched:
                assert len(matched.groups()) == 1
                clean = matched.groups()[0]
        elif remove_affix == "suffix":
            matched = re.search(r"^([\w%]*)" +affix_sep+ r"\w+$", name)
            if matched:
                assert len(matched.groups()) == 1
                clean = matched.groups()[0]
    if matched:
        return clean
    else:
        return None


def remove_labels_affixes(graph, remove_affix, affix_sep = default.affix_sep):
    logger.debug(f"Remove {remove_affix} in labels")
    for uri,p,label in graph.triples((None, RDFS.label, None)):

        if remove_affix != default.remove_affix:
            clean_label = clean_affix_literal(label, remove_affix, affix_sep)

            if clean_label:
                logger.debug(f"Remove {remove_affix} from: {label} to {clean_label}")

                graph.remove((
                    uri,
                    p,
                    label
                ))

                graph.add((
                    uri,
                    p,
                    Literal(clean_label)
                ))
            else:
                logger.debug(f"Label {label} does not need cleaning.")


def remove_affixes_subjects(graph, remove_affix, affix_sep = default.affix_sep):
    logging.debug(f"Remove {remove_affix} from IRIs in subjects")
    if remove_affix != default.remove_affix:
        for uri,p,obj in graph.triples((None, None, None)):
            clean_uri = clean_affix_uri(uri, remove_affix, affix_sep)
            if clean_uri:
                logger.debug(f"Remove subject {remove_affix} from: {uri} to {clean_uri}")

                graph.remove((
                    uri,
                    p,
                    obj
                ))

                graph.add((
                    URIRef(clean_uri),
                    p,
                    obj
                ))
            else:
                logger.debug(f"Subject {uri} does not need cleaning.")


def remove_affixes_objects(graph, remove_affix, affix_sep = default.affix_sep):
    logging.debug(f"Remove {remove_affix} from IRIs in objects")
    if remove_affix != default.remove_affix:
        for uri,p,obj in graph.triples((None, None, None)):
            clean_obj = clean_affix_uri(obj, remove_affix, affix_sep)
            if clean_obj:
                logger.debug(f"Remove object {remove_affix} from: {obj} to {clean_obj}")

                graph.remove((
                    uri,
                    p,
                    obj
                ))

                graph.add((
                    uri,
                    p,
                    URIRef(clean_obj)
                ))
            else:
                logger.debug(f"Object {obj} does not need cleaning.")


def restore_labels(graph, restoration):
    logging.debug("Translate biocypherized labels back")
    for uri,p,label in graph.triples((None, RDFS.label, None)):
        iri = str(uri)
        if iri in restoration:
            logger.info(f"\tRemove {restoration[iri]['biocypher_label']} label from {iri}")
            graph.remove((
                uri,
                RDFS.label,
                rdflib.Literal(restoration[iri]["biocypher_label"])
            ))
            for label in restoration[iri]["origin_labels"]:
                logger.info(f"\tAdd {label} label to {iri}")
                graph.add((
                    uri,
                    rdflib.namespaces.RDFS.label,
                    rdflib.Literal(label)
                ))


def remove_root(graph, root_name = default.root_name):
    logger.debug("Remove the BioCypher root...")
    bcns = Namespace("https://biocypher.org/biocypher#")
    graph.bind("biocypher", bcns)
    namespaces = {}
    for key,ns in graph.namespaces():
        namespaces[key] = Namespace(ns)

    uri_root = URIRef(namespaces["biocypher"][root_name])
    logger.info(f"Remove {root_name} label from {uri_root}")
    graph.remove( (
            uri_root,
            RDFS.label,
            Literal(root_name)
        ) )

    logger.info(f"Remove {uri_root} as a superclass of Thing")
    graph.remove( (
            OWL.Thing,
            RDFS.subClassOf,
            uri_root,
        ) )

    logger.info(f"Remove {uri_root} as a superproperty of topObjectProperty")
    graph.remove( (
            OWL.topObjectProperty,
            RDFS.subPropertyOf,
            uri_root,
        ) )



def restore_owl(graph, restoration, remove_affix = default.remove_affix, affix_sep = default.affix_sep, root_name = default.root_name):
    remove_labels_affixes(graph, remove_affix, affix_sep)
    remove_affixes_subjects(graph, remove_affix, affix_sep)
    remove_affixes_objects(graph, remove_affix, affix_sep)
    restore_labels(graph, restoration)
    remove_root(graph, root_name)
    return graph


if __name__ == "__main__":
    import os
    import sys
    import argparse
    appname = os.path.splitext(os.path.basename(sys.argv[0]))[0]

    do = argparse.ArgumentParser(
        description="Post-process an BioCypherized ontology to restore its original labels.",
        epilog=f"Example usage: {appname} my-biocypherized-onto.owl my-restoration.json > my-restored-onto.owl")

    do.add_argument("ontology")
    do.add_argument("restoration")

    rdflib_formats = ["xml", "n3", "turtle", "nt", "pretty-xml", "trix", "trig", "nquads", "json-ld", "hext"]
    # owlready_formats = ["rdfxml","ntriples"]

    do.add_argument("-f", "--output-format", default="turtle",
        choices=rdflib_formats, metavar="OUT_FORMAT",
        help="the format in which to write the ontology (default: turtle)")

    do.add_argument("-F", "--input-format", default="turtle",
        choices=rdflib_formats, metavar="IN_FORMAT",
        help="the format from which to read the ontology (default: turtle)")

    do.add_argument("-l", "--log-level", default="WARNING",
        choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
        help="Configure the log level. [default: WARNING]")

    do.add_argument("-a", "--type-affix", default=default.remove_affix,
        choices=["suffix","prefix","none"],
        help="Where to add the type string to the ID label.")

    do.add_argument("-A", "--type-affix-sep", default=default.affix_sep,
        metavar="CHARACTER",
        help="Character used to separate the label from the type affix.")

    asked = do.parse_args()

    logging.basicConfig()
    logger.setLevel(asked.log_level)

    logger.debug("Load JSON restoration...")
    with open(asked.restoration) as fd:
        restoration = json.load(fd)

    logger.debug("Load Ontology...")
    graph = rdflib.Graph()

    try:
        graph.parse(source = asked.ontology)
    except Exception as e:
        logger.warning("RDFlib failed to guess the ontology format:\n" \
                       f"`{e}`\n" \
                       f"I'm trying again with format set to: {asked.input_format}")
        graph.parse(source = asked.ontology, format = asked.input_format)

    logger.debug("Restore...")
    restored = restore_owl(graph, restoration, asked.type_affix, asked.type_affix_sep)

    sys.stdout.write(restored.serialize(format = asked.output_format))



""" Interface for transformers' LabelMaker.
"""
import logging
import abc
import re

from . import errormanager, exceptions

logger = logging.getLogger("ontoweaver")

class ReturnCreate:
    """Class for returning the extracted cell value of the transformer, along with the edge type, target node type,
    and properties of the target node type.
    """

    def __init__(self, extracted_cell_value = None, edge_type = None, target_node_type = None, properties_of = None, final_type = None, reverse_relation = None):
        """
        Initializes the ReturnCreate object.

        Args:
            extracted_cell_value: The extracted cell value of the transformer. Can serve as ID of the node, ID of the edge, or value of property.
            edge_type: The type of the edge.
            target_node_type: The type of the target node.
            properties_of: The properties of the returned target node type.
            final_type: The final type of the target node.
            reverse_relation: The reverse relation of the current edge.
        """

        self.extracted_cell_value = extracted_cell_value # Can serve as ID of the node, ID of the edge, or value of property.
        self.edge_type = edge_type
        self.target_node_type = target_node_type
        self.target_element_properties = properties_of
        self.final_type = final_type
        self.reverse_relation = reverse_relation


class LabelMaker(errormanager.ErrorManager, metaclass=abc.ABCMeta):
    """Interface for defining the correct labels (types) for each extracted value.
    The class is responsible for validating the extracted cell value and returning the appropriate label.
    """

    def __init__(self, raise_errors: bool = True):
        """
        Initializes the LabelMaker object.

        Args:
            raise_errors (bool): if True, will raise an exception when an error is encountered, else, will log the error and try to proceed.
        """
        super().__init__(raise_errors)

        self.local_errors = set()
        self.local_warnings = set()

    @abc.abstractmethod
    def __call__(self, validate, returned_value, multi_type_dict, branching_properties = None, row = None):
        """

        Args:
            validate: The validation function which uses the Pandera schema to validate the extracted value.
            returned_value: The extracted value of the cell.
            multi_type_dict: the dictionary holding regex patterns for node and edge type branching based on cell values.
            branching_properties: In case of branching on cell values, the dictionary holding the properties for each branch.
            row: The row of the dataframe that is being processed. Used in case of branching on column values.

        Returns:
            ReturnCreate: An object containing the defined values.
        """
        return NotImplementedError("The call method must be implemented in a subclass.")

    def __del__(self):
        if self.local_errors:
            logger.error("Got errors while making some labels:")
            for e in self.local_errors:
                logger.error(f"\t{e}")

        if self.local_warnings:
            logger.warning(f"Got {len(self.local_warnings)} warnings while making some labels:")
            for i,w in enumerate(self.local_warnings):
                logger.warning(f"\t{i+1}) {w}")


class SimpleLabelMaker(LabelMaker):
    """
    The class is used when the transformer does not have any kind of type branching logic.
    """

    def __init__(self, raise_errors: bool = True):
        super().__init__(raise_errors)

    def __call__(self, validate, returned_value, multi_type_dict, branching_properties = None, row = None):

        value = str(returned_value)
        if validate(value):
            if multi_type_dict:
                if "None" in multi_type_dict.keys():
                    # No branching needed. The transformer is not a branching transformer.
                    assert multi_type_dict["None"]["to_object"]
                    return ReturnCreate(
                        value,
                        multi_type_dict["None"]["via_relation"],
                        multi_type_dict["None"]["to_object"],
                        None,
                        multi_type_dict["None"]["final_type"], multi_type_dict["None"]["reverse_relation"] )
            else:
                # No multi-type dictionary. The transformer returns only the extracted value of the cell. Used for properties.
                return ReturnCreate(value)
        else:
            # Validation failed.
            return None


class MultiTypeLabelMaker(LabelMaker):
    """
    The class is used when the transformer has type branching logic based on the value that will become the ID of the element.
    """
    def __init__(self,raise_errors: bool = True):
        super().__init__(raise_errors)

    def __call__(self, validate, returned_value, multi_type_dict = None, branching_properties = None, row = None):
        label = str(returned_value)
        if validate(label):
            if multi_type_dict:
                has_match = False
                for pattern, types in multi_type_dict.items():
                    # Branching is performed on the regex patterns.
                    if re.search(pattern, label):
                        has_match = True
                        if branching_properties:
                            properties_of = branching_properties.get(types["to_object"].__name__, {})
                        else:
                            properties_of = {}
                        assert types["to_object"]
                        return ReturnCreate(
                            label,
                            types["via_relation"],
                            types["to_object"],
                            properties_of,
                            types["final_type"],
                            types["reverse_relation"]
                        )
                if not has_match:
                    self.local_warnings.add(f"No type pattern matching value: `{label}` during extraction in a match section. This cell value is skipped.")
            else:
                # No multi-type dictionary. The transformer returns only the extracted value of the cell. Used for properties.
                return ReturnCreate(label)
        else:
            # Validation failed, return empty object with None values.
            return ReturnCreate()


class MultiTypeOnColumnLabelMaker(LabelMaker):
    """
    The class is used when the transformer has type branching logic based on the value of a column in the dataframe,
    not the value that will become the ID of the element.
    """
    def __init__(self, raise_errors: bool = True, match_type_from_column: str = None):
        self.match_type_from_column = match_type_from_column
        super().__init__(raise_errors)

    def __call__(self, validate, returned_value, multi_type_dict = None, branching_properties = None, row = None):
        label = str(returned_value)
        if validate(label):
            if multi_type_dict:
                has_match = False
                for pattern, types in multi_type_dict.items():
                    # Branching is performed on the regex patterns.
                    if re.search(pattern, str(row[self.match_type_from_column])):
                        has_match = True
                        if branching_properties:
                            properties_of = branching_properties.get(types["to_object"].__name__, {})
                        else:
                            properties_of = {}
                        assert types["to_object"]
                        return ReturnCreate(
                            label,
                            types["via_relation"],
                            types["to_object"],
                            properties_of,
                            types["final_type"],
                            types["reverse_relation"]
                        )
                if not has_match:
                    self.local_warnings.add(f"No type pattern matching value: `{row[self.match_type_from_column]}` during extraction in a match section. This cell value is skipped.")
            else:
                # No multi-type dictionary. The transformer returns only the extracted value of the cell. Used for properties.
                return ReturnCreate(label)
        else:
            return ReturnCreate()

